from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *

from .tools_counter import *
from .analysis_distribution_one_d import *


class EmceeFitting:

    def __init__(self, input_data, model, initials, limits1, limits2, walkers, iterations, burn_in,
                 names=None, print_names=None, counter=True, counter_window=False, strech_prior=1000.0):

        self.input_data = (np.array(input_data[0]), np.array(input_data[1]))

        self.model = model

        self.initials = np.array(initials)

        self.limits1 = np.array(limits1)
        for i, j in enumerate(limits1):
            if not j:
                limits1[i] = np.nan

        self.limits2 = np.array(limits2)
        for i, j in enumerate(limits2):
            if not j:
                limits2[i] = np.nan

        self.walkers = walkers

        self.iterations = (iterations / walkers) * walkers

        self.burn_in = burn_in

        if names is None:
            self.names = range(1, len(initials) + 1)
        else:
            self.names = names

        if print_names is None:
            self.print_names = map(self.names, str)
        else:
            self.print_names = print_names

        self.counter = counter

        if self.counter is not False:
            if counter is True:
                self.counter = 'MCMC'
            elif isinstance(counter, str):
                self.counter = counter

        self.counter_window = counter_window

        if self.counter_window is not False:
            if counter_window is True:
                self.counter_window = 'MCMC'
            elif isinstance(counter_window, str):
                self.counter_window = counter_window

        self.strech_prior = strech_prior

        self.results = {'input_series': {},
                        'parameters': {},
                        'parameters_final': [],
                        'output_series': {},
                        'statistics': {}}

        self.fitted_parameters = []

        self.mcmc_run_complete = False

    def run_mcmc(self):

        fitted_parameters_indices = np.where(~np.isnan(self.limits1 * self.limits2))[0]

        dimensions = len(fitted_parameters_indices)

        internal_limits1 = self.limits1[fitted_parameters_indices]
        internal_limits2 = self.limits2[fitted_parameters_indices]
        internal_initials = self.initials[fitted_parameters_indices]

        walkers_initial_positions = np.random.uniform(
            (internal_initials -
             (internal_initials - internal_limits1) / self.strech_prior)[:, None] * np.ones(self.walkers),
            (internal_initials +
             (internal_limits2 - internal_initials) / self.strech_prior)[:, None] * np.ones(self.walkers))
        walkers_initial_positions = np.swapaxes(walkers_initial_positions, 0, 1)

        def internal_model(theta):
            parameters = self.initials
            parameters[fitted_parameters_indices] = theta
            return self.model(*parameters)

        def likelihood(theta, data_y, data_y_error):
            if ((internal_limits1 < theta) & (theta < internal_limits2)).all():
                chi = (data_y - internal_model(theta)) / data_y_error
                return -0.5 * (np.sum(chi * chi) + np.sum(np.log(2.0 * np.pi * (data_y_error * data_y_error))))
            else:
                return -np.inf

        def prior(theta):
            if ((internal_limits1 < theta) & (theta < internal_limits2)).all():
                return 0.0
            return -np.inf

        if self.counter or self.counter_window:
            counter = Counter(self.counter, self.counter_window, self.iterations + self.walkers, show_every=100)

            def probability(theta, data_y, data_y_error):
                counter.update()
                return prior(theta) + likelihood(theta, data_y, data_y_error)
        else:
            def probability(theta, data_y, data_y_error):
                return prior(theta) + likelihood(theta, data_y, data_y_error)

        sampler = emcee.EnsembleSampler(self.walkers, dimensions, probability, args=self.input_data)
        sampler.run_mcmc(walkers_initial_positions, int(self.iterations) // int(self.walkers))
        mcmc_results = sampler.flatchain

        self.results['input_series']['value'] = self.input_data[0]
        self.results['input_series']['error'] = self.input_data[1]

        for var in range(len(self.names)):

            if np.isnan(self.limits1[var]):

                variable = {'name': self.names[var], 'print_name': self.print_names[var],
                            'initial': None, 'min_allowed': None, 'max_allowed': None,
                            'trace': None, 'trace_bins': None, 'trace_counts': None,
                            'value': self.initials[var], 'm_error': None, 'p_error': None,
                            'print_value': self.initials[var], 'print_m_error': '-', 'print_p_error': '-'}

            else:
                trace = mcmc_results[:, np.where(fitted_parameters_indices == var)[0][0]]
                trace = trace.reshape(int(self.walkers), int(self.iterations) // int(self.walkers))
                trace = (np.swapaxes(trace, 0, 1).flatten())[self.burn_in:]

                bins, counts, value, m_error, p_error, print_value, print_m_error, print_p_error = \
                    one_d_distribution(trace, confidence_interval=0.68)

                variable = {'name': self.names[var], 'print_name': self.print_names[var],
                            'initial': self.initials[var],
                            'min_allowed': self.limits1[var], 'max_allowed': self.limits2[var],
                            'trace': trace, 'trace_bins': bins, 'trace_counts': counts,
                            'value': value, 'm_error': m_error, 'p_error': p_error,
                            'print_value': print_value, 'print_m_error': print_m_error, 'print_p_error': print_p_error}

                self.fitted_parameters.append(self.names[var])

            self.results['parameters'][self.names[var]] = variable
            self.results['parameters_final'].append(variable['value'])

        self.results['output_series']['model'] = self.model(*self.results['parameters_final'])
        self.results['output_series']['residuals'] = self.input_data[0] - self.results['output_series']['model']

        def correlation(x, y):
            n = len(x)
            mx = np.mean(x)
            sx = np.std(x)
            my = np.mean(y)
            sy = np.std(y)
            return np.round(np.sum((x - mx) * (y - my)) / ((n - 1) * sx * sy), 2)

        correlation_matrix = np.zeros((len(self.fitted_parameters), len(self.fitted_parameters)))

        for ii in range(len(self.fitted_parameters)):
            for jj in range(ii, len(self.fitted_parameters)):
                correlation_matrix[ii][jj] = \
                    correlation(self.results['parameters'][self.fitted_parameters[ii]]['trace'],
                                self.results['parameters'][self.fitted_parameters[jj]]['trace'])

        res_autocorr = np.correlate(self.results['output_series']['residuals'],
                                    self.results['output_series']['residuals'], mode='full')
        res_autocorr = res_autocorr[res_autocorr.size // 2:] / res_autocorr[res_autocorr.size // 2:][0]

        self.results['statistics']['res_autocorr'] = res_autocorr
        self.results['statistics']['res_std'] = np.std(self.results['output_series']['residuals'])
        self.results['statistics']['corr_matrix'] = correlation_matrix
        self.results['statistics']['corr_variables'] = ','.join(self.fitted_parameters)

        self.mcmc_run_complete = True

    def save_all(self, export_file):

        if not self.mcmc_run_complete:
            raise RuntimeError('MCMC not completed')

        pickle.dump(self.results, open(export_file, 'wb'))

    def save_results(self, export_file):

        if not self.mcmc_run_complete:
            raise RuntimeError('MCMC not completed')

        w = open(export_file, 'w')

        w.write('# variable\tresult\tuncertainty\n')

        for i in self.names:
            w.write('{0}\t{1}\t-{2} +{3}\n'.format(self.results['parameters'][i]['name'],
                                                   self.results['parameters'][i]['print_value'],
                                                   self.results['parameters'][i]['print_m_error'],
                                                   self.results['parameters'][i]['print_p_error']))

        w.close()

    def plot_corner(self, export_file):

        def correlation(x, y):
            n = len(x)
            mx = np.mean(x)
            sx = np.std(x)
            my = np.mean(y)
            sy = np.std(y)
            return np.round(np.sum((x - mx) * (y - my)) / ((n - 1) * sx * sy), 2)

        def td_distribution(datax, datay):

            datax = np.array(datax)
            median = np.median(datax)
            med = np.sqrt(np.median((datax - median) ** 2))
            xstep = med / 5.0
            xmin = min(datax)
            xmax = max(datax)
            x_size = int(round((xmax - xmin) / xstep)) + 1
            datax = np.int_((datax - xmin) / xstep)
            datay = np.array(datay)
            median = np.median(datay)
            med = np.sqrt(np.median((datay - median) ** 2))
            ystep = med / 5.0
            ymin = min(datay)
            ymax = max(datay)
            y_size = int(round((ymax - ymin) / ystep)) + 1
            datay = np.int_((datay - ymin) / ystep)

            yx_size = x_size * y_size
            yx = datay * x_size + datax

            yx = np.bincount(yx)
            yx = np.insert(yx, len(yx), np.zeros(yx_size - len(yx)))

            xx, yy = np.meshgrid(xmin + np.arange(x_size) * xstep, ymin + np.arange(y_size) * ystep)

            final = np.reshape(yx, (y_size, x_size))
            plt.imshow(np.where(final > 0, np.log(np.where(final > 0, final, 1)), 0),
                       extent=(np.min(xx), np.max(xx), np.min(yy), np.max(yy)),
                       cmap=plt.cm.Greys, origin='lower', aspect='auto')

        if not self.mcmc_run_complete:
            raise RuntimeError('MCMC not completed')

        names = []
        results = []
        print_results = []
        errors1 = []
        print_errors1 = []
        errors2 = []
        print_errors2 = []
        errors = []
        traces = []
        traces_bins = []
        traces_counts = []

        for i in self.names:
            if self.results['parameters'][i]['initial']:
                names.append(self.results['parameters'][i]['print_name'])
                results.append(self.results['parameters'][i]['value'])
                print_results.append(self.results['parameters'][i]['print_value'])
                errors1.append(self.results['parameters'][i]['m_error'])
                print_errors1.append(self.results['parameters'][i]['print_m_error'])
                errors2.append(self.results['parameters'][i]['p_error'])
                print_errors2.append(self.results['parameters'][i]['print_p_error'])
                errors.append(0.5 * (self.results['parameters'][i]['m_error'] +
                                     self.results['parameters'][i]['p_error']))
                traces.append(self.results['parameters'][i]['trace'])
                traces_bins.append(self.results['parameters'][i]['trace_bins'])
                traces_counts.append(self.results['parameters'][i]['trace_counts'])

        all_var = len(traces)
        fig = plt.figure(figsize=(2.5 * all_var, 2.5 * all_var))
        fig.set_tight_layout(False)
        cmap = matplotlib.cm.get_cmap('brg')

        for var in range(len(names)):

            try:
                plt.subplot(all_var, all_var, all_var * var + var + 1, facecolor='w')
            except AttributeError:
                plt.subplot(all_var, all_var, all_var * var + var + 1, axisbg='w')

            plt.step(traces_bins[var], traces_counts[var], color='k', where='mid')

            plt.axvline(results[var], c='k')
            plt.axvline(results[var] - errors1[var], c='k', ls='--', lw=0.5)
            plt.axvline(results[var] + errors2[var], c='k', ls='--', lw=0.5)

            plt.xticks(plt.xticks()[0], np.ones_like(plt.yticks()[0]))
            plt.yticks(plt.yticks()[0], np.ones_like(plt.yticks()[0]))
            plt.tick_params(left='off', right='off', top='off', bottom='off', labelbottom='off', labelleft='off')

            plt.xlabel('{0}\n{1}\n{2}\n{3}'.format(r'${0}$'.format(names[var]), r'${0}$'.format(print_results[var]),
                                                   r'$-{0}$'.format(print_errors1[var]),
                                                   r'$+{0}$'.format(print_errors2[var])), fontsize=20)

            plt.xlim(results[var] - 6 * errors[var], results[var] + 6 * errors[var])
            plt.ylim(0, plt.ylim()[1])

            for j in range(var + 1, all_var):

                plt.subplot(all_var, all_var, all_var * var + 1 + j)
                td_distribution(traces[j], traces[var])

                plt.yticks(plt.yticks()[0], np.arange(len(plt.yticks()[0])))
                plt.xticks(plt.xticks()[0], np.arange(len(plt.xticks()[0])))
                plt.tick_params(bottom='off', left='off', right='off', top='off', labelbottom='off',
                                labelleft='off', labelright='off', labeltop='off')

                plt.xlim(results[j] - 6 * errors[j], results[j] + 6 * errors[j])
                plt.ylim(results[var] - 6 * errors[var], results[var] + 6 * errors[var])
                text_x = plt.xlim()[1] - 0.05 * (plt.xlim()[1] - plt.xlim()[0])
                text_y = plt.ylim()[1] - 0.05 * (plt.ylim()[1] - plt.ylim()[0])
                plt.text(text_x, text_y, '{0}{1}{2}'.format(r'$', str(correlation(traces[j], traces[var])), '$'),
                         color=cmap(abs(correlation(traces[j], traces[var])) / 2.),
                         fontsize=20, ha='right', va='top')

        plt.subplots_adjust(hspace=0, wspace=0)
        plt.savefig(export_file, bbox_inches='tight', transparent=False)
        plt.close('all')

    def plot_traces(self, export_file):

        if not self.mcmc_run_complete:
            raise RuntimeError('MCMC not completed')

        plt.figure(figsize=(7, 2.5 * len(self.fitted_parameters)))

        for var_num, var in enumerate(self.fitted_parameters):

            plt.subplot(len(self.fitted_parameters), 1, var_num + 1)
            plt.plot(self.results['parameters'][var]['trace'], 'k-', lw=0.1)
            plt.axhline(self.results['parameters'][var]['value'], c='r')
            plt.axhline(self.results['parameters'][var]['value'] - self.results['parameters'][var]['m_error'],
                        ls='--', c='r', lw=0.5)
            plt.axhline(self.results['parameters'][var]['value'] + self.results['parameters'][var]['m_error'],
                        ls='--', c='r', lw=0.5)

            plt.yticks(plt.yticks()[0], np.ones_like(plt.yticks()[0]))
            plt.tick_params(left='off', right='off', labelleft='off')

            plt.ylabel('{0}\n{1}\n{2}\n{3}'.format(r'${0}$'.format(self.results['parameters'][var]['print_name']),
                                                   r'${0}$'.format(self.results['parameters'][var]['print_value']),
                                                   r'$-{0}$'.format(self.results['parameters'][var]['print_m_error']),
                                                   r'$+{0}$'.format(self.results['parameters'][var]['print_p_error'])),
                       fontsize=15)

            if var_num != len(self.fitted_parameters) - 1:
                plt.tick_params(labelbottom='off')
            else:
                plt.xlabel(r'$\mathrm{iteration}$', fontsize=15)

            plt.ylim(self.results['parameters'][var]['value'] - 6 * self.results['parameters'][var]['m_error'],
                     self.results['parameters'][var]['value'] + 6 * self.results['parameters'][var]['m_error'])

        plt.subplots_adjust(hspace=0, wspace=0)
        plt.savefig(export_file, bbox_inches='tight', transparent=True)
        plt.close('all')



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def replace_in_file(file_path, old_string, new_string):
    r = open(str(file_path)).read()
    r = r.replace(str(old_string), str(new_string))
    w = open(str(file_path), 'w')
    w.write(r)
    w.close()


def open_dict(path):

    try:

        return pickle.load(open(path, 'rb'))

    except UnicodeDecodeError:

        return pickle.load(open(path, 'rb'), encoding='latin-1')


def save_dict(dictionary, path):

    internal_copy = copy_dict(dictionary)
    pickle.dump(dictionary, open(path, 'wb'), protocol=2)

    del internal_copy


def copy_dict(dictionary):

    return copy.deepcopy(dictionary)


def open_fits(path):

    with pf.open(path) as hdulist:
        internal_copy = copy_fits(hdulist)

    return internal_copy


def get_fits_arrays(path, index, arrays):

    if isinstance(arrays, str):
        arrays = [arrays]

    out = {}

    with pf.open(path) as fits:
        for array in arrays:
            out[array] = np.ones_like(fits[index].data[array]) * fits[index].data[array]

    return out


def save_fits(hdulist, path):

    internal_copy = copy_fits(hdulist)
    internal_copy.writeto(path, output_verify='fix')

    del internal_copy


def copy_fits(hdulist):

    return pf.HDUList([ff.copy() for ff in hdulist])


def fits_sci(hdulist):

    return np.where(np.array([ff.name for ff in hdulist]) == 'SCI')[0]


def fits_err(hdulist):
    return np.where(np.array([ff.name for ff in hdulist]) == 'ERR')[0]


def fits_sci_err(hdulist):
    return np.swapaxes([fits_sci(hdulist), fits_err(hdulist)], 0, 1)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .analysis_gauss_numerical_integration import *
from .exoplanet_orbit import *


def integral_r_claret(limb_darkening_coefficients, r):
    a1, a2, a3, a4 = limb_darkening_coefficients
    mu44 = 1.0 - r * r
    mu24 = np.sqrt(mu44)
    mu14 = np.sqrt(mu24)
    return - (2.0 * (1.0 - a1 - a2 - a3 - a4) / 4) * mu44 \
           - (2.0 * a1 / 5) * mu44 * mu14 \
           - (2.0 * a2 / 6) * mu44 * mu24 \
           - (2.0 * a3 / 7) * mu44 * mu24 * mu14 \
           - (2.0 * a4 / 8) * mu44 * mu44


def num_claret(r, limb_darkening_coefficients, rprs, z):
    a1, a2, a3, a4 = limb_darkening_coefficients
    rsq = r * r
    mu44 = 1.0 - rsq
    mu24 = np.sqrt(mu44)
    mu14 = np.sqrt(mu24)
    return ((1.0 - a1 - a2 - a3 - a4) + a1 * mu14 + a2 * mu24 + a3 * mu24 * mu14 + a4 * mu44) \
        * r * np.arccos(np.minimum((-rprs ** 2 + z * z + rsq) / (2.0 * z * r), 1.0))


def integral_r_f_claret(limb_darkening_coefficients, rprs, z, r1, r2, precision=3):
    return gauss_numerical_integration(num_claret, r1, r2, precision, limb_darkening_coefficients, rprs, z)


# integral definitions for linear method


def integral_r_linear(limb_darkening_coefficients, r):
    a1 = limb_darkening_coefficients[0]
    musq = 1 - r * r
    return (-1.0 / 6) * musq * (3.0 + a1 * (-3.0 + 2.0 * np.sqrt(musq)))


def num_linear(r, limb_darkening_coefficients, rprs, z):
    a1 = limb_darkening_coefficients[0]
    rsq = r * r
    return (1.0 - a1 * (1.0 - np.sqrt(1.0 - rsq))) \
        * r * np.arccos(np.minimum((-rprs ** 2 + z * z + rsq) / (2.0 * z * r), 1.0))


def integral_r_f_linear(limb_darkening_coefficients, rprs, z, r1, r2, precision=3):
    return gauss_numerical_integration(num_linear, r1, r2, precision, limb_darkening_coefficients, rprs, z)


# integral definitions for quadratic method


def integral_r_quad(limb_darkening_coefficients, r):
    a1, a2 = limb_darkening_coefficients[:2]
    musq = 1 - r * r
    mu = np.sqrt(musq)
    return (1.0 / 12) * (-4.0 * (a1 + 2.0 * a2) * mu * musq + 6.0 * (-1 + a1 + a2) * musq + 3.0 * a2 * musq * musq)


def num_quad(r, limb_darkening_coefficients, rprs, z):
    a1, a2 = limb_darkening_coefficients[:2]
    rsq = r * r
    cc = 1.0 - np.sqrt(1.0 - rsq)
    return (1.0 - a1 * cc - a2 * cc * cc) \
        * r * np.arccos(np.minimum((-rprs ** 2 + z * z + rsq) / (2.0 * z * r), 1.0))


def integral_r_f_quad(limb_darkening_coefficients, rprs, z, r1, r2, precision=3):
    return gauss_numerical_integration(num_quad, r1, r2, precision, limb_darkening_coefficients, rprs, z)


# integral definitions for square root method


def integral_r_sqrt(limb_darkening_coefficients, r):
    a1, a2 = limb_darkening_coefficients[:2]
    musq = 1 - r * r
    mu = np.sqrt(musq)
    return ((-2.0 / 5) * a2 * np.sqrt(mu) - (1.0 / 3) * a1 * mu + (1.0 / 2) * (-1 + a1 + a2)) * musq


def num_sqrt(r, limb_darkening_coefficients, rprs, z):
    a1, a2 = limb_darkening_coefficients[:2]
    rsq = r * r
    mu = np.sqrt(1.0 - rsq)
    return (1.0 - a1 * (1 - mu) - a2 * (1.0 - np.sqrt(mu))) \
        * r * np.arccos(np.minimum((-rprs ** 2 + z * z + rsq) / (2.0 * z * r), 1.0))


def integral_r_f_sqrt(limb_darkening_coefficients, rprs, z, r1, r2, precision=3):
    return gauss_numerical_integration(num_sqrt, r1, r2, precision, limb_darkening_coefficients, rprs, z)


# dictionaries containing the different methods,
# if you define a new method, include the functions in the dictionary as well

integral_r = {
    'claret': integral_r_claret,
    'linear': integral_r_linear,
    'quad': integral_r_quad,
    'sqrt': integral_r_sqrt
}

integral_r_f = {
    'claret': integral_r_f_claret,
    'linear': integral_r_f_linear,
    'quad': integral_r_f_quad,
    'sqrt': integral_r_f_sqrt,
}


def integral_centred(method, limb_darkening_coefficients, rprs, ww1, ww2):
    return (integral_r[method](limb_darkening_coefficients, rprs)
            - integral_r[method](limb_darkening_coefficients, 0.0)) * np.abs(ww2 - ww1)


def integral_plus_core(method, limb_darkening_coefficients, rprs, z, ww1, ww2, precision=3):
    if len(z) == 0:
        return z
    rr1 = z * np.cos(ww1) + np.sqrt(np.maximum(rprs ** 2 - (z * np.sin(ww1)) ** 2, 0))
    rr1 = np.clip(rr1, 0, 1)
    rr2 = z * np.cos(ww2) + np.sqrt(np.maximum(rprs ** 2 - (z * np.sin(ww2)) ** 2, 0))
    rr2 = np.clip(rr2, 0, 1)
    w1 = np.minimum(ww1, ww2)
    r1 = np.minimum(rr1, rr2)
    w2 = np.maximum(ww1, ww2)
    r2 = np.maximum(rr1, rr2)
    parta = integral_r[method](limb_darkening_coefficients, 0.0) * (w1 - w2)
    partb = integral_r[method](limb_darkening_coefficients, r1) * w2
    partc = integral_r[method](limb_darkening_coefficients, r2) * (-w1)
    partd = integral_r_f[method](limb_darkening_coefficients, rprs, z, r1, r2, precision=precision)
    return parta + partb + partc + partd


def integral_minus_core(method, limb_darkening_coefficients, rprs, z, ww1, ww2, precision=3):
    if len(z) == 0:
        return z
    rr1 = z * np.cos(ww1) - np.sqrt(np.maximum(rprs ** 2 - (z * np.sin(ww1)) ** 2, 0))
    rr1 = np.clip(rr1, 0, 1)
    rr2 = z * np.cos(ww2) - np.sqrt(np.maximum(rprs ** 2 - (z * np.sin(ww2)) ** 2, 0))
    rr2 = np.clip(rr2, 0, 1)
    w1 = np.minimum(ww1, ww2)
    r1 = np.minimum(rr1, rr2)
    w2 = np.maximum(ww1, ww2)
    r2 = np.maximum(rr1, rr2)
    parta = integral_r[method](limb_darkening_coefficients, 0.0) * (w1 - w2)
    partb = integral_r[method](limb_darkening_coefficients, r1) * (-w1)
    partc = integral_r[method](limb_darkening_coefficients, r2) * w2
    partd = integral_r_f[method](limb_darkening_coefficients, rprs, z, r1, r2, precision=precision)
    return parta + partb + partc - partd


def transit_flux_drop(method, limb_darkening_coefficients, rp_over_rs, z_over_rs, precision=3):

    if len(z_over_rs) == 0:
        return np.array([])

    z_over_rs = np.where(z_over_rs < 0, 1.0 + 100.0 * rp_over_rs, z_over_rs)

    # cases
    zsq = z_over_rs * z_over_rs
    sum_z_rprs = z_over_rs + rp_over_rs
    dif_z_rprs = rp_over_rs - z_over_rs
    sqr_dif_z_rprs = zsq - rp_over_rs ** 2
    case0 = np.where((z_over_rs == 0) & (rp_over_rs <= 1))
    case1 = np.where((z_over_rs < rp_over_rs) & (sum_z_rprs <= 1))
    casea = np.where((z_over_rs < rp_over_rs) & (sum_z_rprs > 1) & (dif_z_rprs < 1))
    caseb = np.where((z_over_rs < rp_over_rs) & (sum_z_rprs > 1) & (dif_z_rprs > 1))
    case2 = np.where((z_over_rs == rp_over_rs) & (sum_z_rprs <= 1))
    casec = np.where((z_over_rs == rp_over_rs) & (sum_z_rprs > 1))
    case3 = np.where((z_over_rs > rp_over_rs) & (sum_z_rprs < 1))
    case4 = np.where((z_over_rs > rp_over_rs) & (sum_z_rprs == 1))
    case5 = np.where((z_over_rs > rp_over_rs) & (sum_z_rprs > 1) & (sqr_dif_z_rprs < 1))
    case6 = np.where((z_over_rs > rp_over_rs) & (sum_z_rprs > 1) & (sqr_dif_z_rprs == 1))
    case7 = np.where((z_over_rs > rp_over_rs) & (sum_z_rprs > 1) & (sqr_dif_z_rprs > 1) & (-1 < dif_z_rprs))
    plus_case = np.concatenate((case1[0], case2[0], case3[0], case4[0], case5[0], casea[0], casec[0]))
    minus_case = np.concatenate((case3[0], case4[0], case5[0], case6[0], case7[0]))
    star_case = np.concatenate((case5[0], case6[0], case7[0], casea[0], casec[0]))

    # cross points
    ph = np.arccos(np.clip((1.0 - rp_over_rs ** 2 + zsq) / (2.0 * z_over_rs), -1, 1))
    theta_1 = np.zeros(len(z_over_rs))
    ph_case = np.concatenate((case5[0], casea[0], casec[0]))
    theta_1[ph_case] = ph[ph_case]
    theta_2 = np.arcsin(np.minimum(rp_over_rs / z_over_rs, 1))
    theta_2[case1] = np.pi
    theta_2[case2] = np.pi / 2.0
    theta_2[casea] = np.pi
    theta_2[casec] = np.pi / 2.0
    theta_2[case7] = ph[case7]

    # flux_upper
    plusflux = np.zeros(len(z_over_rs))
    plusflux[plus_case] = integral_plus_core(method, limb_darkening_coefficients, rp_over_rs, z_over_rs[plus_case],
                                             theta_1[plus_case], theta_2[plus_case], precision=precision)
    if len(case0[0]) > 0:
        plusflux[case0] = integral_centred(method, limb_darkening_coefficients, rp_over_rs, 0.0, np.pi)
    if len(caseb[0]) > 0:
        plusflux[caseb] = integral_centred(method, limb_darkening_coefficients, 1, 0.0, np.pi)

    # flux_lower
    minsflux = np.zeros(len(z_over_rs))
    minsflux[minus_case] = integral_minus_core(method, limb_darkening_coefficients, rp_over_rs,
                                               z_over_rs[minus_case], 0.0, theta_2[minus_case], precision=precision)

    # flux_star
    starflux = np.zeros(len(z_over_rs))
    starflux[star_case] = integral_centred(method, limb_darkening_coefficients, 1, 0.0, ph[star_case])

    # flux_total
    total_flux = integral_centred(method, limb_darkening_coefficients, 1, 0.0, 2.0 * np.pi)

    return 1 - (2.0 / total_flux) * (plusflux + starflux - minsflux)


def transit(method, limb_darkening_coefficients, rp_over_rs, period, sma_over_rs, eccentricity, inclination, periastron,
            mid_time, time_array, precision=3):

    position_vector = exoplanet_orbit(period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array)

    projected_distance = np.where(
        position_vector[0] < 0, 1.0 + 5.0 * rp_over_rs,
        np.sqrt(position_vector[1] * position_vector[1] + position_vector[2] * position_vector[2]))

    return transit_flux_drop(method, limb_darkening_coefficients, rp_over_rs, projected_distance, precision=precision)


def eclipse(fp_over_fs, rp_over_rs, period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array,
            precision=3):

    position_vector = exoplanet_orbit(period, -sma_over_rs / rp_over_rs, eccentricity, inclination, periastron,
                                      mid_time, time_array)

    projected_distance = np.where(
        position_vector[0] < 0, 1.0 + 5.0 / rp_over_rs,
        np.sqrt(position_vector[1] * position_vector[1] + position_vector[2] * position_vector[2]))

    return (1.0 + fp_over_fs * transit_flux_drop('claret', [0, 0, 0, 0], 1 / rp_over_rs, projected_distance,
                                                 precision=precision)) / (1.0 + fp_over_fs)


def transit_integrated(method, limb_darkening_coefficients, rp_over_rs,
                       period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array,
                       exp_time, time_factor, precision=3):

    exp_time /= (60.0 * 60.0 * 24.0)

    time_factor = int(time_factor)

    time_array_hr = (time_array[:, None] + np.arange(-exp_time / 2 + exp_time / time_factor / 2, exp_time / 2,
                                                     exp_time / time_factor)).flatten()

    position_vector = exoplanet_orbit(period, sma_over_rs, eccentricity,
                                      inclination, periastron, mid_time, time_array_hr)

    projected_distance = np.where(
        position_vector[0] < 0, 1.0 + 5.0 * rp_over_rs,
        np.sqrt(position_vector[1] * position_vector[1] + position_vector[2] * position_vector[2]))

    return np.mean(np.reshape(transit_flux_drop(method, limb_darkening_coefficients,
                                                rp_over_rs, projected_distance, precision=precision),
                              (len(time_array), time_factor)), 1)




from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def correlation(x, y):
    n = len(x)
    mx = np.mean(x)
    sx = np.std(x)
    my = np.mean(y)
    sy = np.std(y)
    return np.round(np.sum((x - mx) * (y - my)) / ((n - 1) * sx * sy), 2)


def waverage(x, x_err):
    x = np.array(x)
    x_err = np.array(x_err)

    wx = 1 / (x_err ** 2)

    wav = np.sum(x * wx, 0) / np.sum(wx, 0)
    wav_err = 1 / np.sqrt((np.sum(wx, 0)))
    return wav, wav_err


def fit_line(datax, datay):

    mx = np.mean(datax)
    my = np.mean(datay)

    ssxx = np.sum((datax - mx) ** 2)
    ssyy = np.sum((datay - my) ** 2)
    ssxy = np.sum((datax - mx) * (datay - my))

    bb = ssxy / ssxx
    aa = my - bb * mx

    n = len(datax)
    sss = np.sqrt((ssyy - bb * ssxy) / (n - 2))

    aerr = sss * np.sqrt(1.0 / n + (mx ** 2) / ssxx)
    berr = sss / np.sqrt(ssxx)

    return aa, bb, aerr, berr


def curve_fit_line(datax, datay, datay_err):

    def fline(x, a, b):
        return a * x+ b

    p0 = np.polyfit(datax, datay, 1)

    popt, pcov = curve_fit(fline, datax, datay, sigma=datay_err, p0=p0)

    return popt[0], popt[1], np.sqrt(pcov[0][0]), np.sqrt(pcov[1][1])


def box(x_arr, x0, aa, bb, cc):
    return aa * np.exp(-np.power(np.power(x0 - x_arr, 2) / (2 * (bb ** 2)), cc))


def fit_box(datax, datay):

    minlim = datax[np.argmax(datay[5:] - datay[:-5])]
    maxlim = datax[np.argmin(datay[5:] - datay[:-5])]

    for expand in range(10, 30):
        datax = np.append(datax, max(datax) + expand)
        datay = np.append(datay, 0)
        datax = np.append(datax, min(datax) - expand)
        datay = np.append(datay, 0)

    popt, pcov = curve_fit(box, datax, datay,
                           p0=[0.5 * (maxlim + minlim), max(datay), 0.5 * (maxlim - minlim), 1.0])

    center = popt[0]

    center_err = np.sqrt(pcov[0][0])

    fwhm = popt[2] * 2.0 * np.sqrt(2.0 * (np.log(2) ** (1.0 / popt[3])))

    s, c = popt[2], popt[3]
    ss, cs = np.sqrt(pcov[2][2]), np.sqrt(pcov[3][3])
    fwhm_err = np.sqrt(8.0 * (ss ** 2) * (np.log(2.0) ** (1.0 / c))
                       + (2.0 * (cs ** 2) * (s ** 2) * (np.log(2.0) ** (1.0 / c))
                          * (np.log(np.log(2.0)) ** 2)) / (c ** 4))

    return center, center_err, fwhm, fwhm_err, popt


def values_to_print(value, error_minus, error_plus):

    try:
        digit1 = abs(int(np.log10(error_minus))) + 1
    except OverflowError:
        digit1 = 3
    try:
        digit2 = abs(int(np.log10(error_plus))) + 1
    except OverflowError:
        digit2 = 3
    try:
        done1 = 1 // int(error_minus * (10 ** digit1))
    except ZeroDivisionError:
        done1 = 0
    try:
        done2 = 1 // int(error_plus * (10 ** digit2))
    except ZeroDivisionError:
        done2 = 0

    if error_minus > 1. and error_plus > 1.:
        digit1, digit2, done1, done2 = 0, 0, 0, 0

    width = max(digit1 + done1, digit2 + done2)

    print_value = '{0:.{width}f}'.format(round(value, width), width=width)
    print_m_error = '{0:.{width}f}'.format(round(error_minus, width), width=width)
    print_p_error = '{0:.{width}f}'.format(round(error_plus, width), width=width)

    return print_value, print_m_error, print_p_error


def find_demicals(number):

    xx = 25

    while round(number, xx) == round(number, xx - 1):
        xx -= 1

    return xx



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .analysis_gauss_numerical_integration import *
from .analysis_distribution_one_d import *


def two_d_distribution(data_array, step=None, min_value=None, max_value=None, confidence_interval=None):

    try:
        data_array = data_array.flatten()
    except AttributeError:
        raise RuntimeError('Invalid input data')

    if not step:
        step = np.sqrt(np.median((data_array - np.median(data_array)) ** 2)) / 5.0
    else:
        try:
            step = float(step)
        except ValueError:
            raise RuntimeError('Invalid step value')

    if not min_value:
        min_value = min(data_array)
    else:
        try:
            min_value = float(min_value)
        except ValueError:
            raise RuntimeError('Invalid minimum value')

    if not max_value:
        max_value = max(data_array)
    else:
        try:
            min_value = float(min_value)
        except ValueError:
            raise RuntimeError('Invalid minimum value')

    bins_number = int((max_value - min_value) / step) + 1

    bins = np.array(min_value + step / 2. + np.arange(bins_number) * step)

    counts = np.bincount(np.int_((data_array - min_value) / step))
    counts = np.insert(counts, len(counts), np.zeros(bins_number - len(counts)))

    if confidence_interval is None:

        return bins, counts

    else:

        try:
            confidence_interval = float(confidence_interval)
        except ValueError:
            raise RuntimeError('Invalid confidence interval value')

        distrx = bins
        bin_width = bins[1] - bins[0]
        distr = counts

        # corresponds to the 1-sigma level probability

        pleft = 0.0
        centroid = np.argmax(distr)
        exp_val = distrx[centroid]

        total_probability_left = np.sum(bin_width * distr[:centroid]) * confidence_interval
        total_probability_right = np.sum(bin_width * distr[centroid:]) * confidence_interval

        num = centroid
        leftci = 0
        while pleft <= total_probability_left:
            if num == centroid:
                pleft += (bin_width / 2.0) * distr[num]
            else:
                pleft += bin_width * distr[num]
            leftci = distrx[num]
            num -= 1
            if num < 0:
                print('ERROR : confidence level can not be reached from left')
                break
        pright = 0.0
        num = centroid
        rightci = 0
        while pright <= total_probability_right:
            if num == centroid:
                pright += (bin_width / 2.0) * distr[num]
            else:
                pright += bin_width * distr[num]
            rightci = distrx[num]
            num += 1
            if num > len(distr) - 1:
                print('ERROR : confidence level can not be reached from right')
                break

        error_plus, error_minus = rightci - exp_val, exp_val - leftci

        try:
            digit1 = abs(int(np.log10(error_minus))) + 1
        except OverflowError:
            digit1 = 3
        try:
            digit2 = abs(int(np.log10(error_plus))) + 1
        except OverflowError:
            digit2 = 3
        try:
            done1 = 1 // int(error_minus * (10 ** digit1))
        except ZeroDivisionError:
            done1 = 0
        try:
            done2 = 1 // int(error_plus * (10 ** digit2))
        except ZeroDivisionError:
            done2 = 0

        if error_minus > 1. and error_plus > 1.:
            digit1, digit2, done1, done2 = 0, 0, 0, 0

        width = max(digit1 + done1, digit2 + done2)

        value = exp_val
        m_error = error_minus
        p_error = error_plus

        print_value = '{0:.{width}f}'.format(round(exp_val, width), width=width)
        print_m_error = '{0:.{width}f}'.format(round(error_minus, width), width=width)
        print_p_error = '{0:.{width}f}'.format(round(error_plus, width), width=width)

        return bins, counts, value, m_error, p_error, print_value, print_m_error, print_p_error


def two_d_gaussian(xy_array, model_norm, model_floor, model_x_mean, model_y_mean, model_x_std, model_y_std,
                   model_theta):

    x_array, y_array = xy_array

    a = (np.cos(model_theta) ** 2) / (2 * model_x_std ** 2) + (np.sin(model_theta) ** 2) / (2 * model_y_std ** 2)
    b = -(np.sin(2 * model_theta)) / (4 * model_x_std ** 2) + (np.sin(2 * model_theta)) / (4 * model_y_std ** 2)
    c = (np.sin(model_theta) ** 2) / (2 * model_x_std ** 2) + (np.cos(model_theta) ** 2) / (2 * model_y_std ** 2)

    return (model_floor + model_norm * np.exp(- (a * ((x_array - model_x_mean) ** 2)
                                              + 2.0 * b * (x_array - model_x_mean) * (y_array - model_y_mean)
                                              + c * ((y_array - model_y_mean) ** 2)))).flatten()


def two_d_gaussian_symmetric(xy_array, model_norm, model_floor, model_x_mean, model_y_mean, model_x_std):

    x_array, y_array = xy_array

    return (model_floor + model_norm * np.exp(-(((model_x_mean - x_array) ** 2) / (model_x_std ** 2)
                                              + ((model_y_mean - y_array) ** 2) / (model_x_std ** 2)) / 2)).flatten()


def two_d_gaussian_positive(xy_array, model_norm, model_floor, model_x_mean, model_y_mean, model_x_std, model_y_std,
                            model_theta):

    return two_d_gaussian(xy_array, np.abs(model_norm), np.abs(model_floor), model_x_mean, model_y_mean, model_x_std,
                          model_y_std, model_theta)


def two_d_gaussian_symmetric_positive(xy_array, model_norm, model_floor, model_x_mean, model_y_mean, model_x_std):

    return two_d_gaussian_symmetric(xy_array, np.abs(model_norm), np.abs(model_floor), model_x_mean, model_y_mean,
                                    model_x_std)


def fit_two_d_gaussian(dataxy, positive=False, symmetric=False, point_xy=None, window=None, sigma=None, maxfev=10000):

    if not point_xy:
        model = gaussian(np.arange(len(dataxy[0])), np.max(dataxy) - np.median(dataxy), np.median(dataxy), len(dataxy[0]) / 2, 1.0)
        dx = np.argmax(np.convolve(np.sum(dataxy, 0), model)) - np.argmax(np.convolve(model, model))
        model = gaussian(np.arange(len(dataxy)), np.max(dataxy) - np.median(dataxy), np.median(dataxy), len(dataxy) / 2, 1.0)
        dy = np.argmax(np.convolve(np.sum(dataxy, 1), model)) - np.argmax(np.convolve(model, model))

        x00 = int(dx + len(dataxy) / 2)
        y00 = int(dy + len(dataxy) / 2)
    else:
        x00, y00 = np.int_(point_xy)

    if window:
        window = np.int(window)
        min_y = int(max(y00 - window, 0))
        max_y = int(min(y00 + window, len(dataxy)))
        min_x = int(max(x00 - window, 0))
        max_x = int(min(x00 + window, len(dataxy[0])))
        dataxy = dataxy[min_y:max_y, min_x:max_x]
    else:
        min_y = 0
        max_y = len(dataxy)
        min_x = 0
        max_x = len(dataxy[0])

    if not sigma:
        sigma = np.abs(np.arange(len(dataxy[0]))[np.argmin(np.abs(np.sum(dataxy, 0)
                                                                  - np.max(np.sum(dataxy, 0)) / 2))] - x00)
    norm = np.max(dataxy) - np.median(dataxy)
    floor = np.median(dataxy)

    if symmetric:
        if positive:
            s_two_d_gaussian = two_d_gaussian_symmetric_positive
        else:
            s_two_d_gaussian = two_d_gaussian_symmetric
        pp0 = [norm, floor, x00, y00, sigma]
    else:
        if positive:
            s_two_d_gaussian = two_d_gaussian_positive
        else:
            s_two_d_gaussian = two_d_gaussian
        pp0 = [norm, floor, x00, y00,  sigma, sigma, 0]

    x = np.arange(min_x, max_x)
    y = np.arange(min_y, max_y)
    x, y = np.meshgrid(x, y)
    popt, pcov = curve_fit(s_two_d_gaussian, (x.flatten() + 0.5, y.flatten() + 0.5), dataxy.flatten(), p0=pp0,
                           maxfev=maxfev)

    return popt, pcov



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys

if sys.version_info[0] > 2:
    from urllib.request import urlretrieve
else:
    from urllib import urlretrieve
    input = raw_input

import matplotlib
if os.environ.get('DISPLAY', '') == '':
    print('no display found. Using non-interactive Agg backend')
    matplotlib.use('Agg')
else:
    try:
        matplotlib.use('TkAgg')
    except ImportError:
        print('matplotlib.pyplot has been already imported. Tk features will not be supported')
        pass

import os
import sys
import copy
import glob
import gzip
import time
import emcee
import numpy as np
import scipy
import pickle
import shutil
import socket
import exodata
import exodata.astroquantities as aq
import datetime
import matplotlib.pyplot as plt
import matplotlib.patches as patches

from scipy.interpolate import griddata
from scipy.optimize import curve_fit
from scipy.interpolate import interp1d
from scipy import interpolate
from astropy.io import fits as pf
from astropy.io import ascii
from astropy.time import Time as astrotime
from astropy.coordinates import get_sun as astrosun
from sklearn.decomposition import FastICA, PCA
from astropy import units as astrounits

from astroquery.simbad import Simbad

if int(matplotlib.__version__[0]) < 3:
    import seaborn
    seaborn.reset_orig()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def initialise_window(window, window_name, windows_to_hide, windows_to_close, exit_python, deactivate_close=False):

    def exit_command():

        for i in windows_to_close:
            i.destroy()

        for i in windows_to_hide:
            i.withdraw()

        if exit_python:
            os._exit(-1)

    if deactivate_close:

        def exit_command():
            pass

    window.wm_title(window_name)
    window.protocol('WM_DELETE_WINDOW', exit_command)

    window.withdraw()


def setup_window(window, objects, main_font=None, button_font=None, entries_bd=3):

    import sys
    if sys.version_info[0] > 2:
        from tkinter import Label
    else:
        from Tkinter import Label

    if button_font is None:
        button_font = ['times', 15, 'bold']

    if main_font is None:
        main_font = ['times', 15]

    for row in range(len(objects)):
        if len(objects[row]) == 0:
            label_empty = Label(window, text='')
            label_empty.grid(row=row, column=100)
        else:
            for obj in objects[row]:

                if obj[0].winfo_class() == 'Button':
                    obj[0].configure(font=button_font)
                elif obj[0].winfo_class() == 'Entry':
                    obj[0].configure(bd=entries_bd, font=main_font)
                elif obj[0].winfo_class() in ['Label', 'Radiobutton']:
                    obj[0].configure(font=main_font)

                if len(obj) == 4:
                    obj[0].grid(row=row, column=obj[1], columnspan=obj[2], rowspan=obj[3])
                elif len(obj) == 3:
                    obj[0].grid(row=row, column=obj[1], columnspan=obj[2])
                else:
                    obj[0].grid(row=row, column=obj[1])


def finalise_window(window, position=5, topmost=False):

    window.update_idletasks()

    if position == 1:
        x = 0
        y = 0

    elif position == 2:
        x = (window.winfo_screenwidth() - window.winfo_reqwidth()) / 2
        y = 0

    elif position == 3:
        x = window.winfo_screenwidth() - window.winfo_reqwidth()
        y = 0

    elif position == 4:
        x = 0
        y = (window.winfo_screenheight() - window.winfo_reqheight()) / 2

    elif position == 5:
        x = (window.winfo_screenwidth() - window.winfo_reqwidth()) / 2
        y = (window.winfo_screenheight() - window.winfo_reqheight()) / 2

    elif position == 6:
        x = window.winfo_screenwidth() - window.winfo_reqwidth()
        y = (window.winfo_screenheight() - window.winfo_reqheight()) / 2

    elif position == 7:
        x = 0
        y = window.winfo_screenheight() - window.winfo_reqheight()

    elif position == 8:
        x = (window.winfo_screenwidth() - window.winfo_reqwidth()) / 2
        y = window.winfo_screenheight() - window.winfo_reqheight()

    elif position == 9:
        x = window.winfo_screenwidth() - window.winfo_reqwidth()
        y = window.winfo_screenheight() - window.winfo_reqheight()

    else:
        x = 0
        y = 0

    window.geometry('+%d+%d' % (x, y))

    window.update_idletasks()

    window.lift()
    window.wm_attributes("-topmost", 1)
    if not topmost:
        window.after_idle(window.attributes, '-topmost', 0)

    window.deiconify()


class Counter():

    def __init__(self, counter, counter_window, total_iterations, show_every=1):

        self.counter = counter

        if self.counter is not False:
            if isinstance(counter, str):
                self.counter = counter
            else:
                self.counter = 'Counter'

        self.counter_window = counter_window

        if self.counter_window is not False:
            if isinstance(counter_window, str):
                self.counter_window = counter_window
            else:
                self.counter_window = 'Counter'

        self.current_iteration = 0
        self.total_iterations = int(total_iterations)
        self.start_time = time.time()
        self.show = 0
        self.show_every = int(show_every)

        if self.total_iterations == 1:
            self.show_every = 10

        if self.counter_window:

            import sys
            if sys.version_info[0] > 2:
                from tkinter import Tk, Label
            else:
                from Tkinter import Tk, Label

            self.root = Tk()

            self.label1 = Label(self.root, text=self.counter_window)
            self.label2 = Label(self.root, text='COMPLETE:')
            self.label3 = Label(self.root, text=' ')
            self.label4 = Label(self.root, text='TIME LEFT:')
            self.label5 = Label(self.root, text=' ')
            self.label6 = Label(self.root, text='TOTAL TIME:')
            self.label7 = Label(self.root, text=' ')

            initialise_window(self.root, self.counter_window, [], [self.root], False, deactivate_close=True)

            setup_window(self.root, [
                [[self.label1, 1, 2]],
                [[self.label2, 1], [self.label3, 2]],
                [[self.label4, 1], [self.label5, 2]],
                [[self.label6, 1], [self.label7, 2]],
            ])

    def update(self, message=''):

        self.current_iteration += 1
        self.show += 1.0 / self.show_every

        out_of = '{0}{1} / {2} {3}'.format(' ' * (len(str(self.total_iterations)) - len(str(self.current_iteration))),
                                       str(self.current_iteration), str(self.total_iterations), message)

        delta_time = time.time() - self.start_time

        time_left = str(datetime.timedelta(
            seconds=int((self.total_iterations - self.current_iteration) * delta_time / self.current_iteration)))

        total_time = str(datetime.timedelta(seconds=int(delta_time)))

        if int(self.show):

            if self.counter_window:
                self.label3.configure(text='{0}{1}{2}'.format('     ', out_of, '     '))
                self.label5.configure(text='{0}{1}{2}'.format('     ', time_left, '     '))
                self.label7.configure(text='{0}{1}{2}'.format('     ', total_time, '     '))
                self.root.update()

            if self.counter:
                sys.stdout.write('\r\033[K')
                sys.stdout.write('{0}{1}: {2}   time left: {3}   total time: {4}'.format(
                    self.counter, '.' * (15 - len(self.counter)), out_of, time_left, total_time))
                sys.stdout.flush()

            self.show = 0

        if self.current_iteration == 1:

            if self.counter_window:
                self.label3.configure(text='{0}{1}{2}'.format('     ', out_of, '     '))
                self.label5.configure(text='{0}{1}{2}'.format('     ', time_left, '     '))
                self.label7.configure(text='{0}{1}{2}'.format('     ', total_time, '     '))
                self.root.update()

                finalise_window(self.root, topmost=True)

                self.root.update()

        elif self.current_iteration == self.total_iterations and self.total_iterations > 1:

            if self.counter_window:
                self.root.destroy()

            if self.counter:
                print('')



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def save_figure(directory, name, figure=None, transparent=True):

    if not os.path.isdir(directory):
        os.makedirs(directory)

    if not figure:
        plt.savefig(directory + '/{0}.eps'.format(name), bbox_inches='tight', transparent=transparent)
        plt.close()
    else:
        figure.savefig(directory + '/{0}.eps'.format(name), bbox_inches='tight', transparent=transparent)


def adjust_ticks():
    xlim1, xlim2 = plt.xlim()
    xticks = plt.xticks()[0]
    digit = max([len(ff) for ff in str(np.abs(xticks) - np.int_(np.abs(xticks)))[1:-1].split()]) - 2
    xticklabels = ['{1:.{0}f}'.format(digit, ii) for ii in xticks]
    dxticks = xticks[1] - xticks[0]

    if xticks[1] - dxticks / 2 < xlim1:
        new_xlim1 = xticks[1] - dxticks / 2
        xticks = xticks[1:]
        xticklabels = xticklabels[1:]
    else:
        new_xlim1 = xticks[0] - dxticks / 2

    if xticks[-2] + dxticks / 2 > xlim2:
        new_xlim2 = xticks[-2] + dxticks / 2
        xticks = xticks[:-1]
        xticklabels = xticklabels[:-1]
    else:
        new_xlim2 = xticks[-1] + dxticks / 2

    plt.xticks(xticks, xticklabels, fontsize=15)
    plt.xlim(new_xlim1, new_xlim2)

    ylim1, ylim2 = plt.ylim()
    yticks = plt.yticks()[0]
    digit = max([len(ff) for ff in str(np.abs(yticks) - np.int_(np.abs(yticks)))[1:-1].split()]) - 2
    yticklabels = ['{1:.{0}f}'.format(digit, ii) for ii in yticks]
    dyticks = yticks[1] - yticks[0]

    if yticks[1] - dyticks / 2 < ylim1:
        new_ylim1 = yticks[1] - dyticks / 2
        yticks = yticks[1:]
        yticklabels = yticklabels[1:]
    else:
        new_ylim1 = yticks[0] - dyticks / 2

    if yticks[-2] + dyticks / 2 > ylim2:
        new_ylim2 = yticks[-2] + dyticks / 2
        yticks = yticks[:-1]
        yticklabels = yticklabels[:-1]
    else:
        new_ylim2 = yticks[-1] + dyticks / 2

    plt.yticks(yticks, yticklabels, fontsize=15)
    plt.ylim(new_ylim1, new_ylim2)


def adjust_ticks_ax(ax):
    xlim1, xlim2 = ax.get_xlim()
    xticks = ax.get_xticks()
    digit = max([len(ff) for ff in str(np.abs(xticks) - np.int_(np.abs(xticks)))[1:-1].split()]) - 2
    xticklabels = ['{1:.{0}f}'.format(digit, ii) for ii in xticks]
    dxticks = xticks[1] - xticks[0]

    if xticks[1] - dxticks / 2 < xlim1:
        new_xlim1 = xticks[1] - dxticks / 2
        xticks = xticks[1:]
        xticklabels = xticklabels[1:]
    else:
        new_xlim1 = xticks[0] - dxticks / 2

    if xticks[-2] + dxticks / 2 > xlim2:
        new_xlim2 = xticks[-2] + dxticks / 2
        xticks = xticks[:-1]
        xticklabels = xticklabels[:-1]
    else:
        new_xlim2 = xticks[-1] + dxticks / 2

    ax.set_xticks(xticks)
    ax.set_xticklabels(xticklabels, fontsize=15)
    ax.set_xlim(new_xlim1, new_xlim2)

    ylim1, ylim2 = ax.get_ylim()
    yticks = ax.get_yticks()
    digit = max([len(ff) for ff in str(np.abs(yticks) - np.int_(np.abs(yticks)))[1:-1].split()]) - 2
    yticklabels = ['{1:.{0}f}'.format(digit, ii) for ii in yticks]
    dyticks = yticks[1] - yticks[0]

    if yticks[1] - dyticks / 2 < ylim1:
        new_ylim1 = yticks[1] - dyticks / 2
        yticks = yticks[1:]
        yticklabels = yticklabels[1:]
    else:
        new_ylim1 = yticks[0] - dyticks / 2

    if yticks[-2] + dyticks / 2 > ylim2:
        new_ylim2 = yticks[-2] + dyticks / 2
        yticks = yticks[:-1]
        yticklabels = yticklabels[:-1]
    else:
        new_ylim2 = yticks[-1] + dyticks / 2

    ax.set_yticks(yticks)
    ax.set_yticklabels(yticklabels, fontsize=15)
    ax.set_ylim(new_ylim1, new_ylim2)




from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def exoplanet_orbit(period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array, ww=0):

    if np.isnan(periastron):
        periastron = 0.
    inclination = inclination * np.pi / 180.0
    periastron = periastron * np.pi / 180.0
    ww = ww * np.pi / 180.0

    if eccentricity == 0 and ww == 0:
        vv = 2 * np.pi * (time_array - mid_time) / period
        bb = sma_over_rs * np.cos(vv)
        return [bb * np.sin(inclination), sma_over_rs * np.sin(vv), - bb * np.cos(inclination)]

    if periastron < np.pi / 2:
        aa = 1.0 * np.pi / 2 - periastron
    else:
        aa = 5.0 * np.pi / 2 - periastron
    bb = 2 * np.arctan(np.sqrt((1 - eccentricity) / (1 + eccentricity)) * np.tan(aa / 2))
    if bb < 0:
        bb += 2 * np.pi
    mid_time = float(mid_time) - (period / 2.0 / np.pi) * (bb - eccentricity * np.sin(bb))
    m = (time_array - mid_time - np.int_((time_array - mid_time) / period) * period) * 2.0 * np.pi / period
    u0 = m
    stop = False
    u1 = 0
    for ii in range(10000):  # setting a limit of 1k iterations - arbitrary limit
        u1 = u0 - (u0 - eccentricity * np.sin(u0) - m) / (1 - eccentricity * np.cos(u0))
        stop = (np.abs(u1 - u0) < 10 ** (-7)).all()
        if stop:
            break
        else:
            u0 = u1
    if not stop:
        raise RuntimeError('Failed to find a solution in 10000 loops')

    vv = 2 * np.arctan(np.sqrt((1 + eccentricity) / (1 - eccentricity)) * np.tan(u1 / 2))
    #
    rr = sma_over_rs * (1 - (eccentricity ** 2)) / (np.ones_like(vv) + eccentricity * np.cos(vv))
    aa = np.cos(vv + periastron)
    bb = np.sin(vv + periastron)
    x = rr * bb * np.sin(inclination)
    y = rr * (-aa * np.cos(ww) + bb * np.sin(ww) * np.cos(inclination))
    z = rr * (-aa * np.sin(ww) - bb * np.cos(ww) * np.cos(inclination))

    return [x, y, z]


def transit_projected_distance(period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array):

    position_vector = exoplanet_orbit(period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array)

    return (np.where(position_vector[0] < 0, -1.0, 1.0) *
            np.sqrt(position_vector[1] * position_vector[1] + position_vector[2] * position_vector[2]))


def transit_duration(rp_over_rs, period, sma_over_rs, inclination, eccentricity, periastron):

    ww = periastron * np.pi / 180
    ii = inclination * np.pi / 180
    ee = eccentricity
    aa = sma_over_rs
    ro_pt = (1 - ee ** 2) / (1 + ee * np.sin(ww))
    b_pt = aa * ro_pt * np.cos(ii)
    if b_pt > 1:
        b_pt = 0.5
    s_ps = 1.0 + rp_over_rs
    df = np.arcsin(np.sqrt((s_ps ** 2 - b_pt ** 2) / ((aa ** 2) * (ro_pt ** 2) - b_pt ** 2)))
    abs_value = (period * (ro_pt ** 2)) / (np.pi * np.sqrt(1 - ee ** 2)) * df

    return abs_value



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def theta_to_phi(th, th0, ph0, a):

    def model(th_m, ph_m):

        return np.cos(a) - (np.cos(th_m) * np.sin(ph_m) * np.cos(th0) * np.sin(ph0) + np.sin(th_m) * np.sin(ph_m) * np.sin(th0) * np.sin(ph0) + np.cos(ph_m) * np.cos(ph0))

    ppov, pcov = curve_fit(model, th, 0)

    return ppov[0]

x=[]
y=[]
z=[]
for th in range(0, 360):
    ph = theta_to_phi(th, 0, np.pi/2, np.pi/10)
    x.append(np.cos(th) * np.sin(ph))
    y.append(np.sin(th) * np.sin(ph))
    z.append(np.cos(ph))

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')
ax.scatter(x,y,z)
plt.show()
# plt.plot(y, z, 'o')
# plt.show()


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def central_crop(original_array, destination_fits):
    crop1 = len(original_array) / 2 - len(destination_fits[1].data) / 2
    crop2 = len(original_array) / 2 + len(destination_fits[1].data) / 2

    return original_array[crop1:crop2, crop1:crop2]


def cartesian_to_polar(x_position, y_position, x_ref_position, y_ref_position):
    x_position, y_position = float(x_position), float(y_position)
    x_ref_position, y_ref_position = float(x_ref_position), float(y_ref_position)

    radius = np.sqrt((x_position - x_ref_position) ** 2 + (y_position - y_ref_position) ** 2)

    if (x_position - x_ref_position) > 0:
        if (y_position - y_ref_position) >= 0:
            angle = np.arctan((y_position - y_ref_position) / (x_position - x_ref_position))
        else:
            angle = 2.0 * np.pi + np.arctan((y_position - y_ref_position) / (x_position - x_ref_position))
    elif (x_position - x_ref_position) < 0:
        angle = np.arctan((y_position - y_ref_position) / (x_position - x_ref_position)) + np.pi
    else:
        if (y_position - y_ref_position) >= 0:
            angle = np.pi / 2
        else:
            angle = -np.pi / 2

    return radius, angle



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._1databases import *


def oec_catalogue():

        oec_file = glob.glob(os.path.join(databases.oec, '*'))[0]

        return exodata.OECDatabase(gzip.GzipFile(oec_file), stream=True)


def find_target(target, catalogue=None):

    if catalogue is None:
        catalogue = oec_catalogue()

    if target == 'Kepler-16 (AB) b':
        planet = catalogue.searchPlanet('Kepler-16 b')
    else:
        planet = catalogue.searchPlanet(target)

    if target == 'XO-1 b':
        planet = planet[1]
    elif isinstance(planet, list):
        planet = planet[0]

    return planet


def find_oec_parameters(target, catalogue=None, binary_star=0):

    planet = find_target(target, catalogue)

    name = planet.name

    try:
        star = planet.star
    except exodata.astroclasses.HierarchyError:
        star = planet.binary.stars[binary_star]

    # known mistakes in the catalogue

    if name in ['HD 3167 b', 'HD 3167 c']:
        star.R = 0.828 * aq.R_s

    stellar_radius = star.R

    stellar_logg = float(star.calcLogg())

    stellar_temperature = float(star.T)

    if np.isnan(star.Z):
        stellar_metallicity = 0
    else:
        stellar_metallicity = star.Z

    rp_over_rs = float(planet.R.rescale(aq.m) / stellar_radius.rescale(aq.m))

    planet_temperature = float(planet.calcTemperature())
    fp_over_fs = (rp_over_rs ** 2.0) * \
                 ((np.exp(6.626 * (10 ** (-5)) * 299792458.0 / (1.4 * 1.381 * stellar_temperature)) - 1.)
                  / (np.exp(6.626 * (10 ** (-5)) * 299792458.0 / (1.4 * 1.381 * planet_temperature)) - 1.))

    period = float(planet.P)

    if np.isnan(planet.a):
        try:
            sma_over_rs = float(planet.calcSMA().rescale(aq.m) / stellar_radius.rescale(aq.m))
        except:
            sma_over_rs = None
    else:
        sma_over_rs = float(planet.a.rescale(aq.m) / stellar_radius.rescale(aq.m))

    if np.isnan(planet.e):
        eccentricity = 0.0
    else:
        eccentricity = float(planet.e)

    if np.isnan(planet.i):
        inclination = 90.0
    else:
        inclination = float(planet.i)

    if np.isnan(planet.periastron):
        periastron = 0.0
    else:
        periastron = float(planet.periastron)

    if np.isnan(planet.transittime):
        mid_time = None
    else:
        mid_time = float(planet.transittime)

    # known mistakes in the catalogue

    if name == 'WASP-121 b':
        mid_time = 2456635.70832
    elif name == 'Qatar-1 b':
        mid_time = 2455518.4102
    elif name == 'HD 209458 b':
        mid_time = 2452826.628521
    elif name == 'KELT-11 b':
        mid_time = 2457061.9098
    elif name == 'WASP-107 b':
        mid_time = 2456514.4106
    elif name == 'WASP-62 b':
        period = 4.411953
        mid_time = 2455855.39195
    elif name == 'HD 3167 b':
        mid_time = 2457394.37450
    elif name == 'HD 3167 c':
        mid_time = 2457394.9788
    elif name == 'WASP-127 b':
        mid_time = 2457248.74126
    elif name == 'HIP 41378 b':
        mid_time = 2457152.2844
    elif name == 'HIP 41378 c':
        mid_time = 2457163.1659
    elif name == 'HIP 41378 d':
        mid_time = 2457166.2629
    elif name == 'HIP 41378 e':
        mid_time = 2457142.01656
    elif name == 'HIP 41378 f':
        mid_time = 2457186.91451
    elif name == 'TrES-2':
        mid_time = 2454955.762517
    elif name == 'HD 97658 b':
        mid_time = 2456665.46415
    elif name == 'Kepler-9 b':
        mid_time += 0.075 * period
    elif name == 'Kepler-9 c':
        period = 39.070
        mid_time = 2456293.581329965 + 0.15
    elif name == 'Kepler-9 d':
        mid_time -= 0.03 * period
    elif name == 'Kepler-11 e':
        period = 31.99590
        mid_time = 2454987.1590
    elif name == 'Kepler-11 d':
        period = 22.68719
        mid_time = 2454981.4550
    elif name == 'KOI-314 c':
        mid_time = 2455558.1404
    elif name == 'HD 219134 b':
        mid_time = 2457463.82884
        period = 3.092926
    elif name == 'HD 106315 b':
        mid_time = 2457605.6521000
    elif name == 'HD 106315 c':
        mid_time = 2457611.1310000
    elif name == 'Kepler-16 (AB) b':
        mid_time = 2455397.522
    elif name == 'CoRoT-1 b':
        mid_time = 2454524.62324
    elif name == 'HAT-P-7 b':
        mid_time = 2454700.81308
    elif name == 'Kepler-1625 b':
        mid_time = 2456043.9587
    elif name == 'WASP-79 b':
        mid_time = 2455545.2356

    return (name, stellar_logg, stellar_temperature, stellar_metallicity, rp_over_rs, fp_over_fs, period, sma_over_rs,
            eccentricity, inclination, periastron, mid_time)


def find_oec_coordinates(target, catalogue=None, output='deg'):

    planet = find_target(target, catalogue)

    if output == 'deg':
        return planet.system.ra.deg, planet.system.dec.deg
    elif output == 'str':
        ra = '{0}:{1}:{2}'.format(str(int(planet.system.ra.hms[0])).zfill(2),
                                  str(int(planet.system.ra.hms[1])).zfill(2),
                                  str(round(planet.system.ra.hms[2],1)).zfill(4))
        if planet.system.dec.deg > 0:
            dec = '+{0}:{1}:{2}'.format(str(int(planet.system.dec.dms[0])).zfill(2),
                                        str(int(planet.system.dec.dms[1])).zfill(2),
                                        str(round(planet.system.dec.dms[2], 1)).zfill(4))
        else:
            dec = '-{0}:{1}:{2}'.format(str(int(abs(planet.system.dec.dms[0]))).zfill(2),
                                        str(int(abs(planet.system.dec.dms[1]))).zfill(2),
                                        str(round(abs(planet.system.dec.dms[2]), 1)).zfill(4))

        return '{0} {1}'.format(ra, dec)


def find_oec_stellar_parameters(target, catalogue=None, binary_star=0):

    planet = find_target(target, catalogue)

    name = planet.name

    try:
        star = planet.star
    except exodata.astroclasses.HierarchyError:
        star = planet.binary.stars[binary_star]

    # known mistakes in the catalogue

    if name in ['HD 3167 b', 'HD 3167 c']:
        star.R = 0.828 * aq.R_s
    try:
        star.R = float(star.R) * aq.R_s
        star.M = float(star.M) * aq.M_s
        stellar_logg = float(star.calcLogg())
    except:
        stellar_logg = None

    stellar_temperature = float(star.T)

    if np.isnan(star.Z):
        stellar_metallicity = 0
    else:
        stellar_metallicity = star.Z

    stellar_radius = float(planet.star.R)

    try:
        magU = float(planet.star.magU)
    except:
        magU = None

    try:
        magB = float(planet.star.magB)
    except:
        magB = None

    try:
        magV = float(planet.star.magV)
    except:
        magV = None

    magR = None
    magI = None

    try:
        magJ = float(planet.star.magJ)
    except:
        magJ = None

    try:
        magH = float(planet.star.magH)
    except:
        magH = None

    try:
        magK = float(planet.star.magK)
    except:
        magK = None

    try:
        magL = float(planet.star.magL)
    except:
        magL = None

    stellar_maglist = [magU, magB, magV, magR, magI, magJ, magH, magK, magL]

    if np.array([mag is None for mag in stellar_maglist]).all():
        stellar_maglist = None

    return name, stellar_logg, stellar_temperature, stellar_metallicity, stellar_radius, stellar_maglist



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .catalogues_oec import *
from .tools_files import *


def ut_to_jd(ut_string):
    tt = astrotime(ut_string)
    return float(tt.jd)


def hjd(ra_target, dec_target, date, time_format):

    date = astrotime(date, format=time_format)
    ra_target *= np.pi / 180
    dec_target *= np.pi / 180

    julian_date = date.jd

    sun = astrosun(date)
    ra_sun, dec_sun = sun.ra.rad, sun.dec.rad

    a = sun.distance.m / 299792458.0
    b = np.sin(dec_target) * np.sin(dec_sun)
    c = np.cos(dec_target) * np.cos(dec_sun) * np.cos(ra_target - ra_sun)

    heliocentric_julian_date = julian_date - (a * (b + c)) / (24.0 * 60.0 * 60.0)

    return heliocentric_julian_date


def jd_to_hjd(ra_target, dec_target, julian_date):

    return hjd(ra_target, dec_target, julian_date, time_format='jd')


def mjd_to_hjd(ra_target, dec_target, modified_julian_date):

    return hjd(ra_target, dec_target, modified_julian_date, time_format='mjd')


def ut_to_hjd(ra_target, dec_target, ut_string):

    return hjd(ra_target, dec_target, ut_string, time_format='iso')


def from_hjd(ra_target, dec_target, heliocentric_julian_date, time_format):

    def func(x, jd):
        return jd_to_hjd(ra_target, dec_target, jd) - heliocentric_julian_date

    popt, pcov = curve_fit(func, [0], [0], p0=[heliocentric_julian_date])
    julian_date = astrotime(popt[0], format='jd')

    if time_format == 'jd':
        return julian_date.jd
    elif time_format == 'mjd':
        return julian_date.mjd
    elif time_format == 'iso':
        return julian_date.iso


def hjd_to_jd(ra_target, dec_target, heliocentric_julian_date):

    return from_hjd(ra_target, dec_target, heliocentric_julian_date, time_format='jd')


def hjd_to_mjd(ra_target, dec_target, heliocentric_julian_date):

    return from_hjd(ra_target, dec_target, heliocentric_julian_date, time_format='mjd')


def hjd_to_ut(ra_target, dec_target, heliocentric_julian_date):

    return from_hjd(ra_target, dec_target, heliocentric_julian_date, time_format='iso')


bjd_dict = open_dict(glob.glob(os.path.join(databases.ephemeris, 'bjd_dict.pickle''*'))[0])
hjd_dict = open_dict(glob.glob(os.path.join(databases.ephemeris, 'hjd_dict.pickle''*'))[0])


def bjd_tdb(ra_target, dec_target, date, time_format):

    date = astrotime(date, format=time_format)
    ra_target *= np.pi / 180
    dec_target *= np.pi / 180

    julian_date = date.jd

    ssb_t = bjd_dict[int(julian_date)]['t']
    ssb_ra = bjd_dict[int(julian_date)]['ra']
    ssb_dec = bjd_dict[int(julian_date)]['dec']
    ssb_d = bjd_dict[int(julian_date)]['d']
    ssb_dt = bjd_dict[int(julian_date)]['dt']

    ssb_ra = interp1d(ssb_t, ssb_ra, kind='cubic')(julian_date)
    ssb_dec = interp1d(ssb_t, ssb_dec, kind='cubic')(julian_date)
    ssb_d = interp1d(ssb_t, ssb_d, kind='cubic')(julian_date)
    ssb_dt = interp1d(ssb_t, ssb_dt, kind='cubic')(julian_date)

    a = ssb_d / 60 / 24
    b = np.sin(dec_target) * np.sin(ssb_dec)
    c = np.cos(dec_target) * np.cos(ssb_dec) * np.cos(ra_target - ssb_ra)

    return julian_date - a * (b + c) + ssb_dt / 60 / 60 / 24


def jd_utc_to_hjd_tdb(ra_target, dec_target, jd_utc):

    ra_target *= np.pi / 180
    dec_target *= np.pi / 180

    sun_t = hjd_dict[int(jd_utc)]['t']
    sun_ra = hjd_dict[int(jd_utc)]['ra']
    sun_dec = hjd_dict[int(jd_utc)]['dec']
    sun_d = hjd_dict[int(jd_utc)]['d']
    sun_dt = hjd_dict[int(jd_utc)]['dt']

    sun_ra = interp1d(sun_t, sun_ra, kind='cubic')(jd_utc)
    sun_dec = interp1d(sun_t, sun_dec, kind='cubic')(jd_utc)
    sun_d = interp1d(sun_t, sun_d, kind='cubic')(jd_utc)
    sun_dt = interp1d(sun_t, sun_dt, kind='cubic')(jd_utc)

    a = sun_d / 60 / 24
    b = np.sin(dec_target) * np.sin(sun_dec)
    c = np.cos(dec_target) * np.cos(sun_dec) * np.cos(ra_target - sun_ra)

    return jd_utc - a * (b + c) + sun_dt / 60 / 60 / 24


def jd_utc_to_hjd_utc(ra_target, dec_target, jd_utc):

    ra_target *= np.pi / 180
    dec_target *= np.pi / 180

    sun_t = hjd_dict[int(jd_utc)]['t']
    sun_ra = hjd_dict[int(jd_utc)]['ra']
    sun_dec = hjd_dict[int(jd_utc)]['dec']
    sun_d = hjd_dict[int(jd_utc)]['d']

    sun_ra = interp1d(sun_t, sun_ra, kind='cubic')(jd_utc)
    sun_dec = interp1d(sun_t, sun_dec, kind='cubic')(jd_utc)
    sun_d = interp1d(sun_t, sun_d, kind='cubic')(jd_utc)

    a = sun_d / 60 / 24
    b = np.sin(dec_target) * np.sin(sun_dec)
    c = np.cos(dec_target) * np.cos(sun_dec) * np.cos(ra_target - sun_ra)

    return jd_utc - a * (b + c)


def jd_utc_to_bjd_tdb(ra_target, dec_target, jd_utc):

    return bjd_tdb(ra_target, dec_target, jd_utc, time_format='jd')


def mjd_utc_to_bjd_tdb(ra_target, dec_target, mjd_utc):

    return bjd_tdb(ra_target, dec_target, mjd_utc, time_format='mjd')


def utc_to_bjd_tdb(ra_target, dec_target, utc):

    return bjd_tdb(ra_target, dec_target, utc, time_format='iso')


def hjd_utc_to_bjd_tdb(ra_target, dec_target, hjd_utc):

    def func(x, jd):
        return jd_utc_to_hjd_utc(ra_target, dec_target, jd) - hjd_utc

    popt, pcov = curve_fit(func, [0], [0], p0=[hjd_utc])

    return jd_utc_to_bjd_tdb(ra_target, dec_target, popt[0])


def hjd_tdb_to_bjd_tdb(ra_target, dec_target, hjd_tdb):

    def func(x, jd):
        return jd_utc_to_hjd_tdb(ra_target, dec_target, jd) - hjd_tdb

    popt, pcov = curve_fit(func, [0], [0], p0=[hjd_tdb])

    return jd_utc_to_bjd_tdb(ra_target, dec_target, popt[0])


def bjd_utc_to_bjd_tdb(ra_target, dec_target, bjd_utc):

    ssb_t = bjd_dict[int(bjd_utc)]['t']
    ssb_dt = bjd_dict[int(bjd_utc)]['dt']
    ssb_dt = interp1d(ssb_t, ssb_dt, kind='cubic')(bjd_utc)

    return bjd_utc + ssb_dt / 60 / 60 / 24


def bjd_tdb_to_jd_utc(ra_target, dec_target, bjd_tdb_in):

    def func(x, jd):
        return jd_utc_to_bjd_tdb(ra_target, dec_target, jd) - bjd_tdb_in

    popt, pcov = curve_fit(func, [0], [0], p0=[bjd_tdb_in])

    return popt[0]


def ra_dec_string_to_deg(ra_dec_string):

    if len(ra_dec_string.split()) != 2:
        if len(ra_dec_string.split()) == 6:
            ra_dec_string = '{}:{}:{} {}:{}:{}'.format(*ra_dec_string.split())
        else:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[0].split(':')) != 3:
        print('Wrong RA-DEC format.')
        return None, None

    if len(ra_dec_string.split()[0].split(':')[0]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(ra_dec_string.split()[0].split(':')[0]) >= 24:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[0].split(':')[1]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(ra_dec_string.split()[0].split(':')[1]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[0].split(':')[2]) < 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if float(ra_dec_string.split()[0].split(':')[2]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[1][1:].split(':')) != 3:
        print('Wrong RA-DEC format.')
        return None, None

    if len(ra_dec_string.split()[1].split(':')[0]) != 3:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if abs(int(ra_dec_string.split()[1].split(':')[0])) >= 90:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[1].split(':')[1]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(ra_dec_string.split()[1].split(':')[1]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(ra_dec_string.split()[1].split(':')[2]) < 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if float(ra_dec_string.split()[1].split(':')[2]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    try:
        ra_h, ra_m, ra_s = ra_dec_string.split()[0].split(':')
        ra = float(ra_h) / 1.0 + float(ra_m) / 60.0 + float(ra_s) / 3600.0
        ra *= 15.0
        dec_d, dec_m, dec_s = ra_dec_string.split()[1][1:].split(':')
        if ra_dec_string.split()[1][0] == '+':
            dec = float(dec_d) / 1.0 + float(dec_m) / 60.0 + float(dec_s) / 3600.0
        elif ra_dec_string.split()[1][0] == '-':
            dec = - float(dec_d) / 1.0 - float(dec_m) / 60.0 - float(dec_s) / 3600.0
        else:
            print('Wrong RA-DEC format.')
            return None, None

        return ra, dec

    except (ValueError, TypeError):
        print('Wrong RA-DEC format.')
        return None, None


def lat_long_string_to_deg(lat_long_string):

    if len(lat_long_string.split()) != 2:
        if len(lat_long_string.split()) == 6:
            lat_long_string = '{}:{}:{} {}:{}:{}'.format(*lat_long_string.split())
        else:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[0].split(':')) != 3:
        print('Wrong RA-DEC format.')
        return None, None

    if len(lat_long_string.split()[0].split(':')[0]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(lat_long_string.split()[0].split(':')[0]) >= 24:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[0].split(':')[1]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(lat_long_string.split()[0].split(':')[1]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[0].split(':')[2]) < 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if float(lat_long_string.split()[0].split(':')[2]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[1][1:].split(':')) != 3:
        print('Wrong RA-DEC format.')
        return None, None

    if len(lat_long_string.split()[1].split(':')[0]) != 3:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if abs(int(lat_long_string.split()[1].split(':')[0])) >= 90:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[1].split(':')[1]) != 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if int(lat_long_string.split()[1].split(':')[1]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    if len(lat_long_string.split()[1].split(':')[2]) < 2:
        print('Wrong RA-DEC format.')
        return None, None
    else:
        try:
            if float(lat_long_string.split()[1].split(':')[2]) >= 60:
                print('Wrong RA-DEC format.')
                return None, None
        except ValueError:
            print('Wrong RA-DEC format.')
            return None, None

    try:
        lat_d, lat_m, lat_s = lat_long_string.split()[0].split(':')
        lat = float(lat_d) / 1.0 + float(lat_m) / 60.0 + float(lat_s) / 3600.0
        long_d, long_m, long_s = lat_long_string.split()[1][1:].split(':')
        if lat_long_string.split()[1][0] == '+':
            long = float(long_d) / 1.0 + float(long_m) / 60.0 + float(long_s) / 3600.0
        elif lat_long_string.split()[1][0] == '-':
            long = - float(long_d) / 1.0 - float(long_m) / 60.0 - float(long_s) / 3600.0
        else:
            print('Wrong RA-DEC format.')
            return None, None

        return lat, long

    except (ValueError, TypeError):
        print('Wrong LAT-LONG format.')
        return None, None




#
# def find_next_transit(target, date, catalogue=None):
#
#     planet = find_target(target, catalogue)
#
#     (planet_name, stellar_logg, stellar_temperature, stellar_metallicity, rp_over_rs, fp_over_fs,
#      period, sma_over_rs, eccentricity, inclination, periastron, mid_time) = find_oec_parameters(target, catalogue)
#
#     date_hjd = jd_to_hjd(ephem.Date(date) + 2415020, planet.system.ra.deg, planet.system.dec.deg)
#
#     next_date_hjd = mid_time + (int((date_hjd - mid_time) / period) + 1) * period
#
#     next_date_dif = (next_date_hjd - date_hjd) * 24.0
#
#     next_date = ephem.Date(ephem.Date(date) + (next_date_hjd - date_hjd))
#
#     return planet_name, next_date_dif, '{0}/{1}/{2} {3}:{4}:{5:.0f}'.format(*next_date.tuple())
#
#
# def find_current_phase(target, julian_date, catalogue=None):
#
#     planet = find_target(target, catalogue)
#
#     (planet_name, stellar_logg, stellar_temperature, stellar_metallicity, rp_over_rs, fp_over_fs,
#      period, sma_over_rs, eccentricity, inclination, periastron, mid_time) = find_oec_parameters(target, catalogue)
#
#     if julian_date < mid_time:
#         mid_time -= int((mid_time - julian_date)/period + 10) * period
#
#     date_hjd = jd_to_hjd(julian_date, planet.system.ra.deg, planet.system.dec.deg)
#
#     return (date_hjd - mid_time) / period - int((date_hjd - mid_time) / period)


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._0errors import *
from ._0imports import *
from .tools_databases import *


class Databases:

    def __init__(self):

        self.clablimb = Database('clablimb', 'pylightcurve', __file__, date_to_update='191023').path
        self.ephemeris = Database('ephemeris', 'pylightcurve', __file__, date_to_update='200303').path
        self.oec = Database('oec', 'pylightcurve', __file__, date_to_update='daily').path

    def phoenix(self):
        return Database('phoenixplc3', 'pylightcurve', __file__, date_to_update='181213', ask_size='3.5GB').path


databases = Databases()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from ._0errors import *
from ._0imports import *


class Database:

    def __init__(self, database_name, package_name, file, vital=False, date_to_update='daily', force_update=False,
                 ask_size=None, database_file_name='_0database.pickle'):

        directory_name = 'database'
        last_update_file_name = 'database_last_update.txt'

        info_file_path = os.path.join(os.path.abspath(os.path.dirname(file)), database_file_name)
        package_path = os.path.join(os.path.expanduser('~'), '.{0}'.format(package_name))
        if not os.path.isdir(package_path):
            os.mkdir(package_path)
        directory_path = os.path.join(package_path, '{0}_{1}'.format(database_name, directory_name))
        last_update_file_path = os.path.join(package_path, '{0}_{1}'.format(database_name, last_update_file_name))

        if date_to_update == 'daily':
            date_to_update = int(time.strftime('%y%m%d'))
        else:
            date_to_update = int(date_to_update)

        if os.path.isdir(directory_path):
            if force_update or len(glob.glob(os.path.join(directory_path, '*'))) == 0:
                shutil.rmtree(directory_path)
                os.mkdir(directory_path)
                update = True
            else:
                if not os.path.isfile(last_update_file_path):
                    update = True
                elif int(open(last_update_file_path).readlines()[0]) < date_to_update:
                    update = True
                else:
                    update = False
        else:
            os.mkdir(directory_path)
            update = True

        if update and ask_size:
            if input('Downloading {0} database (up to {1})... proceed with download now? (y/n): '.format(
                    database_name, ask_size)) == 'y':
                update = True
            else:
                update = False

        if update:
            # noinspection PyBroadException
            try:
                print('\nDownloading {0} database...'.format(database_name))

                dbx_files = pickle.load(open(info_file_path, 'rb'))
                dbx_files = dbx_files['{0}_{1}'.format(database_name, directory_name)]

                for current_file in glob.glob(os.path.join(directory_path, '*')):
                    if os.path.split(current_file)[1] not in dbx_files:
                        os.remove(current_file)

                if database_name == 'oec':
                    for i in dbx_files:
                        dbx_files[i]['link'] = \
                            'https://github.com/OpenExoplanetCatalogue/oec_gzip/raw/master/systems.xml.gz'

                for dbx_file in dbx_files:
                    if not os.path.isfile(os.path.join(package_path, dbx_files[dbx_file]['local_path'])):
                        print(dbx_file)
                        urlretrieve(dbx_files[dbx_file]['link'], os.path.join(package_path,
                                                                              dbx_files[dbx_file]['local_path']))

                if database_name == 'clablimb':
                    xx = pickle.load(open(glob.glob(os.path.join(directory_path, '*.pickle'))[0], 'rb'))
                    for i in xx:
                        w = open(os.path.join(directory_path, i), 'w')
                        w.write(xx[i])
                        w.close()

                w = open(last_update_file_path, 'w')
                w.write(time.strftime('%y%m%d'))
                w.close()

            except Exception as inst:
                print('\nDownloading {0} database failed. A download will be attempted next time.'.format(
                    database_name))
                print('Error:', sys.exc_info()[0])
                print(inst.args)
                pass

        if (not os.path.isdir(directory_path) or
                len(glob.glob(os.path.join(directory_path, '*'))) == 0):
            if vital:
                raise PyLCLibraryError('{0} database not available.'.format(database_name))
            else:
                print('\n{0} features cannot be used.'.format(database_name))
                self.path = False
        else:
            self.path = directory_path



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


class PyLCError(BaseException):
    pass


class PyLCLibraryError(PyLCError):
    pass


class PyLCFileError(PyLCError):
    pass


class PyLCProcessError(PyLCError):
    pass


class PyLCInputError(PyLCError):
    pass



from setuptools import setup, Extension
import os
import sys
import glob
import time
import codecs
import shutil
import socket
import numpy

if sys.version_info[0] > 2:
    from urllib.request import urlopen, urlretrieve
else:
    from urllib import urlopen, urlretrieve
    input = raw_input

name = 'pylightcurve'
description = 'A python package for modeling and analysing transit light-curves.'
url = 'https://https://github.com/ucl-exoplanets/pylightcurve'
install_requires = ['matplotlib', 'numpy', 'exodata', 'emcee', 'seaborn', 'astropy', 'scipy', 'sklearn', 'astroquery']

os.chdir(os.path.abspath(os.path.dirname(__file__)))

subdirs_to_include = []
for x in os.walk(name):
    if os.path.isdir(x[0]):
        if x[0] != name:
            subdirs_to_include.append(x[0])

files_to_include = []
for x in glob.glob(os.path.join(name, '*')):
    if os.path.isfile(x):
        if x.split('.')[-1] not in ['py']:
            files_to_include.append(os.path.join(name, os.path.split(x)[1]))

files_to_include.append('README.md')
files_to_include.append('LICENSE')
files_to_include.append('readme.md')
files_to_include.append('licence')

w = open('MANIFEST.in', 'w')
for i in subdirs_to_include:
    w.write('include ' + os.path.join(i, '*') + ' \n')

for i in files_to_include:
    w.write('include ' + i + ' \n')

w.close()

version = ' '
for i in open(os.path.join(name, '__init__.py')):
    if len(i.split('__version__')) > 1:
        version = i.split()[-1][1:-1]

setup(
    name=name,
    version=version,
    description=description,
    long_description='Visit https://github.com/ucl-exoplanets/pylightcurve',
    url=url,
    author='Angelos Tsiaras',
    author_email='aggelostsiaras@gmail.com',
    license='MIT',
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Console',
                 'Intended Audience :: Science/Research',
                 'Topic :: Scientific/Engineering :: Astronomy',
                 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                 'Operating System :: MacOS :: MacOS X',
                 'Programming Language :: Python :: 3.7',
                 ],
    packages=[name],
    install_requires=install_requires,
    include_package_data=True,
    zip_safe=False,
)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .tools_files import *


def get_vega_flux(mag):

    zmag = None
    phot_filter = None
    if mag == 'U':
        zmag = 4.175e-9
        phot_filter = 'Bessel_U.txt'

    if mag == 'B':
        zmag = 6.32e-9
        phot_filter = 'Bessel_B.txt'

    if mag == 'V':
        zmag = 3.631e-9
        phot_filter = 'Bessel_V.txt'

    if mag == 'R':
        zmag = 2.177e-9
        phot_filter = 'Bessel_R.txt'

    if mag == 'I':
        zmag = 1.126e-9
        phot_filter = 'Bessel_I.txt'

    if mag == 'J':
        zmag = 3.147e-10
        phot_filter = 'Bessel_J.txt'

    if mag == 'H':
        zmag = 1.138e-10
        phot_filter = 'Bessel_H.txt'

    if mag == 'K':
        zmag = 9.961e-11
        phot_filter = 'Bessel_K.txt'

    if mag == 'L':
        zmag = 7.08e-12
        phot_filter = 'Bessel_L.txt'

    if mag == 'G':
        zmag = 2.4773273088492937e-09
        phot_filter = 'G.txt'

    wavelength = pf.open('/Users/angelos/Desktop/alpha_lyr_004.fits')[1].data['WAVELENGTH']
    flux = pf.open('/Users/angelos/Desktop/alpha_lyr_004.fits')[1].data['FLUX']

    phot_filter = np.loadtxt(os.path.join(databases.phoenix(), phot_filter), unpack=True)
    phot_filter[0] = phot_filter[0] * 10
    band = interp1d(phot_filter[0], np.maximum(0, phot_filter[1]), kind='cubic')
    test = np.where((wavelength > min(phot_filter[0])) * (wavelength < max(phot_filter[0])))
    wavelength_test = wavelength[test]
    flux_test = flux[test]

    flux = np.sum(((flux_test * band(wavelength_test) * wavelength_test)[:-1] *
                     (wavelength_test[1:] - wavelength_test[:-1]))) / np.sum(
        (band(wavelength_test) * wavelength_test)[:-1] * (wavelength_test[1:] - wavelength_test[:-1]))

    return flux


def open_spectrum_file(fits):

    fits = get_fits_arrays(os.path.join(databases.phoenix(), fits), 1, ['Wavelength', 'Flux'])
    wavelength_array = fits['Wavelength']
    flux_array = fits['Flux']

    return wavelength_array * 10000, flux_array / 10
    # units: wavelength: A, flux: erg/s/cm^2/A


def get_spectrum(stellar_logg, stellar_temperature, stellar_radius,
                 stellar_umag=None, stellar_bmag=None, stellar_vmag=None,
                 stellar_rmag=None, stellar_imag=None,
                 stellar_jmag=None, stellar_hmag=None, stellar_kmag=None,
                 stellar_lmag=None, stellar_gmag=None):

    grid = open_dict(os.path.join(databases.phoenix(), 'grid.pickle'))

    logg = list(grid.keys())[np.argmin((np.array(list(grid.keys())) - stellar_logg) ** 2)]

    temperatures = list(grid[logg])
    temperatures.sort()

    temperature = temperatures[np.argmin((np.array(temperatures) - stellar_temperature) ** 2)]

    if temperature == stellar_temperature and logg == stellar_logg:

        wavelength1, flux = open_spectrum_file(grid[logg][temperature])

    else:

        loggs = list(grid.keys())
        loggs.sort()

        logg = loggs[np.argmin((np.array(loggs) - stellar_logg) ** 2)]
        logg1 = loggs[loggs.index(logg) - 1]
        logg2 = loggs[loggs.index(logg) + 1]

        # logg1

        temperatures = list(grid[logg1])
        temperatures.sort()

        temperature = temperatures[np.argmin((np.array(temperatures) - stellar_temperature) ** 2)]
        temperature1 = temperatures[temperatures.index(temperature) - 1]
        temperature2 = temperatures[temperatures.index(temperature) + 1]

        wavelength1, flux1 = open_spectrum_file(grid[logg][temperature1])
        wavelength2, flux2 = open_spectrum_file(grid[logg][temperature2])

        flux_logg1 = flux1 + (flux2 - flux1) * (stellar_temperature - temperature1) / (temperature2 - temperature1)

        # logg2

        temperature = temperatures[np.argmin((np.array(temperatures) - stellar_temperature) ** 2)]
        temperature1 = temperatures[temperatures.index(temperature) - 1]
        temperature2 = temperatures[temperatures.index(temperature) + 1]

        wavelength1, flux1 = open_spectrum_file(grid[logg][temperature1])
        wavelength2, flux2 = open_spectrum_file(grid[logg][temperature2])

        flux_logg2 = flux1 + (flux2 - flux1) * (stellar_temperature - temperature1) / (temperature2 - temperature1)

        # final

        flux = flux_logg1 + (flux_logg2 - flux_logg1) * (stellar_logg - logg1) / (logg2 - logg1)

    wavelength = wavelength1

    zmag = False
    stellar_mag = False
    phot_filter = False

    # Bessel 1990, PASP 102, 1181

    if isinstance(stellar_umag, float) or isinstance(stellar_umag, int):
        zmag = 4.175e-9
        phot_filter = 'Bessel_U.txt'
        stellar_mag = stellar_umag

    if isinstance(stellar_bmag, float) or isinstance(stellar_bmag, int):
        zmag = 6.32e-9
        phot_filter = 'Bessel_B.txt'
        stellar_mag = stellar_bmag

    if isinstance(stellar_vmag, float) or isinstance(stellar_vmag, int):
        zmag = 3.631e-9
        phot_filter = 'Bessel_V.txt'
        stellar_mag = stellar_vmag

    if isinstance(stellar_rmag, float) or isinstance(stellar_rmag, int):
        zmag = 2.177e-9
        phot_filter = 'Bessel_R.txt'
        stellar_mag = stellar_rmag

    if isinstance(stellar_imag, float) or isinstance(stellar_imag, int):
        zmag = 1.126e-9
        phot_filter = 'Bessel_I.txt'
        stellar_mag = stellar_imag

    if isinstance(stellar_jmag, float) or isinstance(stellar_jmag, int):
        zmag = 3.147e-10
        phot_filter = 'Bessel_J.txt'
        stellar_mag = stellar_jmag

    if isinstance(stellar_hmag, float) or isinstance(stellar_hmag, int):
        zmag = 1.138e-10
        phot_filter = 'Bessel_H.txt'
        stellar_mag = stellar_hmag

    if isinstance(stellar_kmag, float) or isinstance(stellar_kmag, int):
        zmag = 9.961e-11
        phot_filter = 'Bessel_K.txt'
        stellar_mag = stellar_kmag

    if isinstance(stellar_lmag, float) or isinstance(stellar_lmag, int):
        zmag = 7.08e-12
        phot_filter = 'Bessel_L.txt'
        stellar_mag = stellar_lmag

    if isinstance(stellar_gmag, float) or isinstance(stellar_gmag, int):
        zmag = 2.4773273088492937e-09
        phot_filter = 'G.txt'
        stellar_mag = stellar_gmag

    if zmag:

        phot_filter = np.loadtxt(os.path.join(databases.phoenix(), phot_filter), unpack=True)
        phot_filter[0] = phot_filter[0] * 10
        band = interp1d(phot_filter[0], np.maximum(0, phot_filter[1]), kind='cubic')
        test = np.where((wavelength > min(phot_filter[0])) * (wavelength < max(phot_filter[0])))
        wavelength_test = wavelength[test]
        flux_test = flux[test]

        f_flux = np.sum(((flux_test * band(wavelength_test) * wavelength_test)[:-1] *
                         (wavelength_test[1:] - wavelength_test[:-1]))) / np.sum(
            (band(wavelength_test) * wavelength_test)[:-1] * (wavelength_test[1:] - wavelength_test[:-1]))

        # stellar_radius *= 695700

        factor = zmag / (10 ** (stellar_mag / 2.5)) / f_flux

        return wavelength, flux * factor

    else:
        return wavelength, flux


def get_flux(stellar_logg, stellar_temperature, stellar_radius, lambda1, lambda2,
             stellar_umag=None, stellar_bmag=None, stellar_vmag=None,
             stellar_rmag=None, stellar_imag=None,
             stellar_jmag=None, stellar_hmag=None, stellar_kmag=None,
             stellar_lmag=None, stellar_gmag=None):

    wavelength_array, flux_array = get_spectrum(stellar_logg, stellar_temperature, stellar_radius,
                                                stellar_umag, stellar_bmag, stellar_vmag, stellar_rmag,
                                                stellar_imag, stellar_jmag, stellar_hmag, stellar_kmag,
                                                stellar_lmag, stellar_gmag)

    binsedge = 0.5 * (wavelength_array[:-1] + wavelength_array[1:])
    binsedge1 = np.append(wavelength_array[0] - (binsedge[0] - wavelength_array[0]), binsedge)
    binsedge2 = np.append(binsedge, wavelength_array[-1] + (wavelength_array[-1] - binsedge[-1]))
    binswidth = binsedge2 - binsedge1
    arg1 = np.where((lambda1 >= binsedge1) * (lambda1 < binsedge2))[0][0]
    arg2 = np.where((lambda2 > binsedge1) * (lambda2 <= binsedge2))[0][0]

    flux = np.sum(binswidth[arg1 + 1: arg2] * flux_array[arg1 + 1: arg2])
    flux += flux_array[arg1] * (binsedge2[arg1]-lambda1)
    flux += flux_array[arg2] * (lambda2 - binsedge1[arg2])

    return flux


def get_mag(stellar_logg, stellar_temperature, stellar_radius, distance, mag):

    # Bessel 1990, PASP 102, 1181
    zmag = None
    phot_filter = None
    if mag == 'U':
        zmag = 4.175e-9
        phot_filter = 'Bessel_U.txt'

    if mag == 'B':
        zmag = 6.32e-9
        phot_filter = 'Bessel_B.txt'

    if mag == 'V':
        zmag = 3.631e-9
        phot_filter = 'Bessel_V.txt'

    if mag == 'R':
        zmag = 2.177e-9
        phot_filter = 'Bessel_R.txt'

    if mag == 'I':
        zmag = 1.126e-9
        phot_filter = 'Bessel_I.txt'

    if mag == 'J':
        zmag = 3.147e-10
        phot_filter = 'Bessel_J.txt'

    if mag == 'H':
        zmag = 1.138e-10
        phot_filter = 'Bessel_H.txt'

    if mag == 'K':
        zmag = 9.961e-11
        phot_filter = 'Bessel_K.txt'

    if mag == 'L':
        zmag = 7.08e-12
        phot_filter = 'Bessel_L.txt'

    if mag == 'G':
        zmag = 2.4773273088492937e-09
        phot_filter = 'G.txt'

    if not zmag:
        raise PyLCInputError('Filter not available.')
    else:
        band = np.loadtxt(os.path.join(databases.phoenix(), phot_filter), unpack=True)
        band[0] = band[0] * 10

        ffcnc = get_spectrum(stellar_logg, stellar_temperature, stellar_radius)
        test = np.where((ffcnc[0] > min(band[0])) * (ffcnc[0] < max(band[0])))
        ffcnc = [ffcnc[0][test], ffcnc[1][test]]

        band = interp1d(band[0], band[1], kind='cubic')

        flux = np.sum((ffcnc[1] * band(ffcnc[0]) * ffcnc[0])[:-1] * (ffcnc[0][1:] - ffcnc[0][:-1])) / np.sum(
            (band(ffcnc[0]) * ffcnc[0])[:-1] * (ffcnc[0][1:] - ffcnc[0][:-1]))

        stellar_radius *= 695700
        distance *= 3.08567758149137e+13

        return 2.5 * np.log10(zmag/(flux * (stellar_radius * stellar_radius / distance / distance)))


def g_to_r_mag(stellar_logg, stellar_temperature, stellar_radius, stellar_mag):

    wavelength, flux = get_spectrum(stellar_logg, stellar_temperature, stellar_radius, stellar_gmag=stellar_mag)
    phot_filter = 'G.txt'

    phot_filter = np.loadtxt(os.path.join(databases.phoenix(), phot_filter), unpack=True)
    phot_filter[0] = phot_filter[0] * 10
    band = interp1d(phot_filter[0], np.maximum(0, phot_filter[1]), kind='cubic')
    test = np.where((wavelength > min(phot_filter[0])) * (wavelength < max(phot_filter[0])))
    wavelength_test = wavelength[test]
    flux_test = flux[test]

    flux = np.sum(((flux_test * band(wavelength_test) * wavelength_test)[:-1] *
                     (wavelength_test[1:] - wavelength_test[:-1]))) / np.sum(
        (band(wavelength_test) * wavelength_test)[:-1] * (wavelength_test[1:] - wavelength_test[:-1]))

    return 2.5 * np.log10(2.2090695086397724e-09/flux)


def g_to_v_mag(stellar_logg, stellar_temperature, stellar_radius, stellar_mag):

    wavelength, flux = get_spectrum(stellar_logg, stellar_temperature, stellar_radius, stellar_gmag=stellar_mag)
    phot_filter = 'G.txt'

    phot_filter = np.loadtxt(os.path.join(databases.phoenix(), phot_filter), unpack=True)
    phot_filter[0] = phot_filter[0] * 10
    band = interp1d(phot_filter[0], np.maximum(0, phot_filter[1]), kind='cubic')
    test = np.where((wavelength > min(phot_filter[0])) * (wavelength < max(phot_filter[0])))
    wavelength_test = wavelength[test]
    flux_test = flux[test]

    flux = np.sum(((flux_test * band(wavelength_test) * wavelength_test)[:-1] *
                     (wavelength_test[1:] - wavelength_test[:-1]))) / np.sum(
        (band(wavelength_test) * wavelength_test)[:-1] * (wavelength_test[1:] - wavelength_test[:-1]))

    return 2.5 * np.log10(3.519416551428086e-09/flux)


def clablimb(method, stellar_logg, stellar_temperature, stellar_metallicity, photometric_filter, stellar_model='ATLAS'):

    if method != 'claret':
        raise ImportError('Limb darkening model not currently supported.')

    data_file = glob.glob(os.path.join(databases.clablimb,
                                       '*_{0}_{1}.txt'.format(stellar_model, photometric_filter)))[0]
    data = np.loadtxt(data_file, usecols=[0, 1, 2, 3, 4, 5, 6, 7], unpack=True)

    xx = np.unique(data[0])
    yy = np.unique(data[1])
    zz = np.unique(data[2])

    yin = float(stellar_temperature)
    zin = float(stellar_metallicity)
    xin = float(stellar_logg)

    if xin in xx:
        xmin, xmax = xin, xin
    else:
        test = np.argmin(np.abs(np.ones_like(xx) * xin - xx))
        if xx[test] < xin:
            xmin = xx[test]
            xmax = xx[min(len(xx) - 1, test + 1)]
        else:
            xmin = xx[max(0, test - 1)]
            xmax = xx[test]
    if yin in yy:
        ymin, ymax = yin, yin
    else:
        test = np.argmin(np.abs(np.ones_like(yy) * yin - yy))
        if yy[test] < yin:
            ymin = yy[test]
            ymax = yy[min(len(yy) - 1, test + 1)]
        else:
            ymin = yy[max(0, test - 1)]
            ymax = yy[test]
    if zin in zz:
        zmin, zmax = zin, zin
    else:
        test = np.argmin(np.abs(np.ones_like(zz) * zin - zz))
        if zz[test] < zin:
            zmin = zz[test]
            zmax = zz[min(len(zz) - 1, test + 1)]
        else:
            zmin = zz[max(0, test - 1)]
            zmax = zz[test]

    def tri_linear(x, y, z, x0, x1, y0, y1, z0, z1, v000, v100, v010, v001, v101, v011, v110, v111):
        c0 = v000
        c1 = v100 - v000
        c2 = v010 - v000
        c3 = v001 - v000
        c4 = v110 - v010 - v100 + v000
        c5 = v011 - v001 - v010 + v000
        c6 = v101 - v001 - v100 + v000
        c7 = v111 - v011 - v101 - v110 + v100 + v001 + v010 - v000
        if x == x0 == x1:
            dx = 0
        else:
            dx = (x - x0) / (x1 - x0)
        if y == y0 == y1:
            dy = 0
        else:
            dy = (y - y0) / (y1 - y0)
        if z == z0 == z1:
            dz = 0
        else:
            dz = (z - z0) / (z1 - z0)
        return c0 + c1 * dx + c2 * dy + c3 * dz + c4 * dx * dy + c5 * dy * dz + c6 * dz * dx + c7 * dx * dy * dz

    final_coefficients = []

    for index in [4, 5, 6, 7]:

        vv000 = data[index][np.where((data[0] == xmin) & (data[1] == ymin) & (data[2] == zmin))][0]
        vv100 = data[index][np.where((data[0] == xmax) & (data[1] == ymin) & (data[2] == zmin))][0]
        vv010 = data[index][np.where((data[0] == xmin) & (data[1] == ymax) & (data[2] == zmin))][0]
        vv001 = data[index][np.where((data[0] == xmin) & (data[1] == ymin) & (data[2] == zmax))][0]
        vv101 = data[index][np.where((data[0] == xmax) & (data[1] == ymin) & (data[2] == zmax))][0]
        vv011 = data[index][np.where((data[0] == xmin) & (data[1] == ymax) & (data[2] == zmax))][0]
        vv110 = data[index][np.where((data[0] == xmax) & (data[1] == ymax) & (data[2] == zmin))][0]
        vv111 = data[index][np.where((data[0] == xmax) & (data[1] == ymax) & (data[2] == zmax))][0]

        res = tri_linear(xin, yin, zin, xmin, xmax, ymin, ymax, zmin, zmax,
                         vv000, vv100, vv010, vv001, vv101, vv011, vv110, vv111)

        final_coefficients.append(res)

    return np.array(final_coefficients)
